#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import uuid
import json
import zipfile


from tests.utils import asyncio_patch, AsyncioMagicMock

from gns3server.controller.project import Project
from gns3server.controller.import_project import import_project, _move_files_to_compute

from gns3server.version import __version__


def test_import_project(async_run, tmpdir, controller):
    project_id = str(uuid.uuid4())

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "auto_open": True,
        "auto_start": True,
        "topology": {
        },
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)
    with open(str(tmpdir / "b.png"), 'w+') as f:
        f.write("B")

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")
        myzip.write(str(tmpdir / "b.png"), "b.png")
        myzip.write(str(tmpdir / "b.png"), "project-files/dynamips/test")
        myzip.write(str(tmpdir / "b.png"), "project-files/qemu/test")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    assert project.name == "test"
    assert project.id == project_id

    assert os.path.exists(os.path.join(project.path, "b.png"))
    assert not os.path.exists(os.path.join(project.path, "project.gns3"))
    assert os.path.exists(os.path.join(project.path, "test.gns3"))
    assert os.path.exists(os.path.join(project.path, "project-files/dynamips/test"))
    assert os.path.exists(os.path.join(project.path, "project-files/qemu/test"))

    # A new project name is generated when you import twice the same name
    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, str(uuid.uuid4()), f))
    assert project.auto_open is False
    assert project.auto_start is False
    assert project.name != "test"


def test_import_project_override(async_run, tmpdir, controller):
    """
    In the case of snapshot we will import a project for
    override the previous keeping the same project id & location
    """
    project_id = str(uuid.uuid4())

    topology = {
        "project_id": project_id,
        "name": "test",
        "topology": {
        },
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f, location=str(tmpdir)))

    assert project.name == "test"
    assert project.id == project_id

    # Overide the project with same project
    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f, location=str(tmpdir)))
    assert project.id == project_id
    assert project.name == "test"


def test_import_upgrade(async_run, tmpdir, controller):
    """
    Topology made for previous GNS3 version are upgraded during the process
    """
    project_id = str(uuid.uuid4())

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "topology": {
        },
        "version": "1.4.2"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        assert topo["version"] == __version__


def test_import_with_images(tmpdir, async_run, controller):

    project_id = str(uuid.uuid4())

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "topology": {
        },
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    with open(str(tmpdir / "test.image"), 'w+') as f:
        f.write("B")

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")
        myzip.write(str(tmpdir / "test.image"), "images/IOS/test.image")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    assert not os.path.exists(os.path.join(project.path, "images/IOS/test.image"))

    path = os.path.join(project._config().get("images_path"), "IOS", "test.image")
    assert os.path.exists(path), path


def test_import_iou_linux_no_vm(linux_platform, async_run, tmpdir, controller):
    """
    On non linux host IOU should be local if we don't have a GNS3 VM
    """
    project_id = str(uuid.uuid4())

    controller._computes["local"] = AsyncioMagicMock()

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "type": "topology",
        "topology": {
            "nodes": [
                {
                    "compute_id": "local",
                    "node_type": "iou",
                    "name": "test",
                    "properties": {}
                }
            ],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "revision": 5,
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        assert topo["topology"]["nodes"][0]["compute_id"] == "local"


def test_import_iou_linux_with_vm(linux_platform, async_run, tmpdir, controller):
    """
    On non linux host IOU should be vm if we have a GNS3 VM configured
    """
    project_id = str(uuid.uuid4())
    controller._computes["vm"] = AsyncioMagicMock()

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "type": "topology",
        "topology": {
            "nodes": [
                {
                    "compute_id": "local",
                    "node_id": "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b",
                    "node_type": "iou",
                    "name": "test",
                    "properties": {}
                }
            ],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "revision": 5,
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        assert topo["topology"]["nodes"][0]["compute_id"] == "vm"


def test_import_nat_non_linux(windows_platform, async_run, tmpdir, controller):
    """
    On non linux host NAT should be moved to the GNS3 VM
    """
    project_id = str(uuid.uuid4())
    controller._computes["vm"] = AsyncioMagicMock()

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "type": "topology",
        "topology": {
            "nodes": [
                {
                    "compute_id": "local",
                    "node_id": "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b",
                    "node_type": "nat",
                    "name": "test",
                    "properties": {}
                }
            ],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "revision": 5,
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        assert topo["topology"]["nodes"][0]["compute_id"] == "vm"


def test_import_iou_non_linux(windows_platform, async_run, tmpdir, controller):
    """
    On non linux host IOU should be moved to the GNS3 VM
    """
    project_id = str(uuid.uuid4())
    controller._computes["vm"] = AsyncioMagicMock()

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "type": "topology",
        "topology": {
            "nodes": [
                {
                    "compute_id": "local",
                    "node_id": "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b",
                    "node_type": "iou",
                    "name": "test",
                    "properties": {}
                },
                {
                    "compute_id": "local",
                    "node_type": "vpcs",
                    "name": "test2",
                    "properties": {}
                }
            ],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "revision": 5,
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        with asyncio_patch("gns3server.controller.import_project._move_files_to_compute") as mock:
            project = async_run(import_project(controller, project_id, f))
            controller._computes["vm"].post.assert_called_with('/projects', data={'name': 'test', 'project_id': project_id})

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        assert topo["topology"]["nodes"][0]["compute_id"] == "vm"
        assert topo["topology"]["nodes"][1]["compute_id"] == "local"

    mock.assert_called_with(controller._computes["vm"], project_id, project.path, os.path.join('project-files', 'iou', topo["topology"]["nodes"][0]['node_id']))


def test_import_node_id(linux_platform, async_run, tmpdir, controller):
    """
    When importing a node, node_id should change
    """
    project_id = str(uuid.uuid4())

    controller._computes["local"] = AsyncioMagicMock()

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "type": "topology",
        "topology": {
            "nodes": [
                {
                    "compute_id": "local",
                    "node_id": "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b",
                    "node_type": "iou",
                    "name": "test",
                    "properties": {}
                },
                {
                    "compute_id": "local",
                    "node_id": "c3ae286c-c81f-40d9-a2d0-5874b2f2478d",
                    "node_type": "iou",
                    "name": "test2",
                    "properties": {}
                }
            ],
            "links": [
                {
                    "link_id": "b570a150-c09f-47d9-8d32-9ca5b03234d6",
                    "nodes": [
                        {
                            "adapter_number": 0,
                            "node_id": "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b",
                            "port_number": 0
                        },
                        {
                            "adapter_number": 0,
                            "node_id": "c3ae286c-c81f-40d9-a2d0-5874b2f2478d",
                            "port_number": 0
                        }
                    ]
                }
            ],
            "computes": [],
            "drawings": [
                {
                    "drawing_id": "08d665ba-e982-4d54-82b4-aa0c4d5ba6a3",
                    "rotation": 0,
                    "x": -210,
                    "y": -108,
                    "z": 0
                }
            ]
        },
        "revision": 5,
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    # Fake .gns3project
    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")
        myzip.writestr("project-files/iou/0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b/startup.cfg", "test")
        myzip.writestr("project-files/iou/c3ae286c-c81f-40d9-a2d0-5874b2f2478d/startup.cfg", "test")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f))

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        # Node id should have change
        assert topo["topology"]["nodes"][0]["node_id"] not in ["0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b", "c3ae286c-c81f-40d9-a2d0-5874b2f2478d"]

        # Link should have change
        link = topo["topology"]["links"][0]
        assert link["link_id"] != "b570a150-c09f-47d9-8d32-9ca5b03234d6"
        assert link["nodes"][0]["node_id"] not in ["0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b", "c3ae286c-c81f-40d9-a2d0-5874b2f2478d"]

        # Drawing id should change
        assert topo["topology"]["drawings"][0]["drawing_id"] != "08d665ba-e982-4d54-82b4-aa0c4d5ba6a3"

        # Node files should have moved to the new node id
        assert not os.path.exists(os.path.join(project.path, "project-files", "iou", "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b", "startup.cfg"))
        assert not os.path.exists(os.path.join(project.path, "project-files", "iou", "c3ae286c-c81f-40d9-a2d0-5874b2f2478d", "startup.cfg"))
        assert os.path.exists(os.path.join(project.path, "project-files", "iou", topo["topology"]["nodes"][0]["node_id"], "startup.cfg"))


def test_import_keep_compute_id(windows_platform, async_run, tmpdir, controller):
    """
    On linux host IOU should be moved to the GNS3 VM
    """
    project_id = str(uuid.uuid4())
    controller._computes["vm"] = AsyncioMagicMock()

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "type": "topology",
        "topology": {
            "nodes": [
                {
                    "compute_id": "local",
                    "node_id": "0fd3dd4d-dc93-4a04-a9b9-7396a9e22e8b",
                    "node_type": "iou",
                    "name": "test",
                    "properties": {}
                }
            ],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "revision": 5,
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f, keep_compute_id=True))

    with open(os.path.join(project.path, "test.gns3")) as f:
        topo = json.load(f)
        assert topo["topology"]["nodes"][0]["compute_id"] == "local"


def test_move_files_to_compute(tmpdir, async_run):
    project_id = str(uuid.uuid4())

    os.makedirs(str(tmpdir / "project-files" / "docker"))
    (tmpdir / "project-files" / "docker" / "test").open("w").close()
    (tmpdir / "project-files" / "docker" / "test2").open("w").close()

    with asyncio_patch("gns3server.controller.import_project._upload_file") as mock:
        async_run(_move_files_to_compute(None, project_id, str(tmpdir), os.path.join("project-files", "docker")))

    mock.assert_any_call(None, project_id, str(tmpdir / "project-files" / "docker" / "test"), os.path.join("project-files", "docker", "test"))
    mock.assert_any_call(None, project_id, str(tmpdir / "project-files" / "docker" / "test2"), os.path.join("project-files", "docker", "test2"))
    assert not os.path.exists(str(tmpdir / "project-files" / "docker"))


def test_import_project_name_and_location(async_run, tmpdir, controller):
    """
    Import a project with a different location and name
    """
    project_id = str(uuid.uuid4())

    topology = {
        "project_id": str(uuid.uuid4()),
        "name": "test",
        "topology": {
        },
        "version": "2.0.0"
    }

    with open(str(tmpdir / "project.gns3"), 'w+') as f:
        json.dump(topology, f)

    zip_path = str(tmpdir / "project.zip")
    with zipfile.ZipFile(zip_path, 'w') as myzip:
        myzip.write(str(tmpdir / "project.gns3"), "project.gns3")

    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, project_id, f, name="hello", location=str(tmpdir / "hello")))

    assert project.name == "hello"

    assert os.path.exists(str(tmpdir / "hello" / "hello.gns3"))

    # A new project name is generated when you import twice the same name
    with open(zip_path, "rb") as f:
        project = async_run(import_project(controller, str(uuid.uuid4()), f, name="hello", location=str(tmpdir / "test")))
    assert project.name == "hello-1"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VirtualBox server module.
"""

import os
import sys
import shutil
import asyncio
import subprocess
import logging

log = logging.getLogger(__name__)

from ..base_manager import BaseManager
from .virtualbox_vm import VirtualBoxVM
from .virtualbox_error import VirtualBoxError


class VirtualBox(BaseManager):

    _NODE_CLASS = VirtualBoxVM

    def __init__(self):

        super().__init__()
        self._vboxmanage_path = None
        self._execute_lock = asyncio.Lock()

    @property
    def vboxmanage_path(self):
        """
        Returns the path to VBoxManage.

        :returns: path
        """

        return self._vboxmanage_path

    def find_vboxmanage(self):

        # look for VBoxManage
        vboxmanage_path = self.config.get_section_config("VirtualBox").get("vboxmanage_path")
        if vboxmanage_path:
            if not os.path.isabs(vboxmanage_path):
                vboxmanage_path = shutil.which(vboxmanage_path)
        else:
            log.info("A path to VBoxManage has not been configured, trying to find it...")
            if sys.platform.startswith("win"):
                if "VBOX_INSTALL_PATH" in os.environ:
                    vboxmanage_path_windows = os.path.join(os.environ["VBOX_INSTALL_PATH"], "VBoxManage.exe")
                    if os.path.exists(vboxmanage_path_windows):
                        vboxmanage_path = vboxmanage_path_windows
                elif "VBOX_MSI_INSTALL_PATH" in os.environ:
                    vboxmanage_path_windows = os.path.join(os.environ["VBOX_MSI_INSTALL_PATH"], "VBoxManage.exe")
                    if os.path.exists(vboxmanage_path_windows):
                        vboxmanage_path = vboxmanage_path_windows
            elif sys.platform.startswith("darwin"):
                vboxmanage_path_osx = "/Applications/VirtualBox.app/Contents/MacOS/VBoxManage"
                if os.path.exists(vboxmanage_path_osx):
                    vboxmanage_path = vboxmanage_path_osx
            if not vboxmanage_path:
                vboxmanage_path = shutil.which("vboxmanage")

        if vboxmanage_path and not os.path.exists(vboxmanage_path):
            log.error("VBoxManage path '{}' doesn't exist".format(vboxmanage_path))

        if not vboxmanage_path:
            raise VirtualBoxError("Could not find VBoxManage, please reboot if VirtualBox has just been installed")
        if not os.path.isfile(vboxmanage_path):
            raise VirtualBoxError("VBoxManage '{}' is not accessible".format(vboxmanage_path))
        if not os.access(vboxmanage_path, os.X_OK):
            raise VirtualBoxError("VBoxManage is not executable")
        if os.path.basename(vboxmanage_path) not in ["VBoxManage", "VBoxManage.exe", "vboxmanage"]:
            raise VirtualBoxError("Invalid VBoxManage executable name {}".format(os.path.basename(vboxmanage_path)))

        self._vboxmanage_path = vboxmanage_path
        return vboxmanage_path

    @asyncio.coroutine
    def execute(self, subcommand, args, timeout=60):

        # We use a lock prevent parallel execution due to strange errors
        # reported by a user and reproduced by us.
        # https://github.com/GNS3/gns3-gui/issues/261
        with (yield from self._execute_lock):
            vboxmanage_path = self.vboxmanage_path
            if not vboxmanage_path:
                vboxmanage_path = self.find_vboxmanage()
            if not vboxmanage_path:
                raise VirtualBoxError("Could not find VBoxManage")

            command = [vboxmanage_path, "--nologo", subcommand]
            command.extend(args)
            command_string = " ".join(command)
            log.info("Executing VBoxManage with command: {}".format(command_string))
            try:
                process = yield from asyncio.create_subprocess_exec(*command, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE)
            except (OSError, subprocess.SubprocessError) as e:
                raise VirtualBoxError("Could not execute VBoxManage: {}".format(e))

            try:
                stdout_data, stderr_data = yield from asyncio.wait_for(process.communicate(), timeout=timeout)
            except asyncio.TimeoutError:
                raise VirtualBoxError("VBoxManage has timed out after {} seconds!".format(timeout))

            if process.returncode:
                vboxmanage_error = stderr_data.decode("utf-8", errors="ignore")
                raise VirtualBoxError("VirtualBox has returned an error: {}".format(vboxmanage_error))

            return stdout_data.decode("utf-8", errors="ignore").splitlines()

    @asyncio.coroutine
    def _find_inaccessible_hdd_files(self):
        """
        Finds inaccessible disk files (to clean up the VirtualBox media manager)
        """

        hdds = []
        try:
            properties = yield from self.execute("list", ["hdds"])
        # If VirtualBox is not available we have no inaccessible hdd
        except VirtualBoxError:
            return hdds

        flag_inaccessible = False
        for prop in properties:
            try:
                name, value = prop.split(':', 1)
            except ValueError:
                continue
            if name.strip() == "State" and value.strip() == "inaccessible":
                flag_inaccessible = True
            if flag_inaccessible and name.strip() == "Location":
                hdds.append(value.strip())
                flag_inaccessible = False
        return reversed(hdds)

    @asyncio.coroutine
    def project_closed(self, project):
        """
        Called when a project is closed.

        :param project: Project instance
        """

        yield from super().project_closed(project)
        hdd_files_to_close = yield from self._find_inaccessible_hdd_files()
        for hdd_file in hdd_files_to_close:
            log.info("Closing VirtualBox VM disk file {}".format(os.path.basename(hdd_file)))
            try:
                yield from self.execute("closemedium", ["disk", hdd_file])
            except VirtualBoxError as e:
                log.warning("Could not close VirtualBox VM disk file {}: {}".format(os.path.basename(hdd_file), e))
                continue

    @asyncio.coroutine
    def list_vms(self, allow_clone=False):
        """
        Gets VirtualBox VM list.
        """

        vbox_vms = []
        result = yield from self.execute("list", ["vms"])
        for line in result:
            if len(line) == 0 or line[0] != '"' or line[-1:] != "}":
                continue  # Broken output (perhaps a carriage return in VM name)
            vmname, _ = line.rsplit(' ', 1)
            vmname = vmname.strip('"')
            if vmname == "<inaccessible>":
                continue  # ignore inaccessible VMs
            extra_data = yield from self.execute("getextradata", [vmname, "GNS3/Clone"])
            if allow_clone or len(extra_data) == 0 or not extra_data[0].strip() == "Value: yes":
                # get the amount of RAM
                info_results = yield from self.execute("showvminfo", [vmname, "--machinereadable"])
                ram = 0
                for info in info_results:
                    try:
                        name, value = info.split('=', 1)
                        if name.strip() == "memory":
                            ram = int(value.strip())
                            break
                    except ValueError:
                        continue
                vbox_vms.append({"vmname": vmname, "ram": ram})
        return vbox_vms

    @staticmethod
    def get_legacy_vm_workdir(legacy_vm_id, name):
        """
        Returns the name of the legacy working directory name for a node.

        :param legacy_vm_id: legacy node identifier (not used)
        :param name: node name

        :returns: working directory name
        """

        return os.path.join("vbox", "{}".format(name))



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pytest
import aiohttp
import asyncio
from unittest.mock import MagicMock


from gns3server.controller.link import Link
from gns3server.controller.node import Node
from gns3server.controller.ports.ethernet_port import EthernetPort
from gns3server.controller.ports.serial_port import SerialPort
from gns3server.controller.compute import Compute
from gns3server.controller.project import Project

from tests.utils import AsyncioBytesIO, AsyncioMagicMock


@pytest.fixture
def project(controller):
    return Project(controller=controller, name="Test")


@pytest.fixture
def compute():
    return Compute("example.com", controller=MagicMock())


@pytest.fixture
def link(async_run, project, compute):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]
    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 1, 3)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))
    async_run(link.add_node(node2, 1, 3))
    return link


def test_eq(project, link, controller):
    assert link == Link(project, link_id=link.id)
    assert link != "a"
    assert link != Link(project)


def test_add_node(async_run, project, compute):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()
    project.dump = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))
    assert link._nodes == [
        {
            "node": node1,
            "port": node1._ports[0],
            "adapter_number": 0,
            "port_number": 4,
            'label': {
                'y': -10,
                'text': '0/4',
                'x': -10,
                'rotation': 0,
                'style': 'font-size: 10; font-style: Verdana'
            }
        }
    ]
    assert project.dump.called
    assert not link._project.controller.notification.emit.called

    assert not link.create.called

    # We call link.created only when both side are created
    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]
    async_run(link.add_node(node2, 0, 4))

    assert link.create.called
    link._project.controller.notification.emit.assert_called_with("link.created", link.__json__())
    assert link in node2.links


def test_add_node_already_connected(async_run, project, compute):
    """
    Raise an error if we try to use an already connected port
    """
    project.dump = AsyncioMagicMock()

    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()
    async_run(link.add_node(node1, 0, 4))
    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]
    async_run(link.add_node(node2, 0, 4))

    assert link.create.called
    link2 = Link(project)
    link2.create = AsyncioMagicMock()
    with pytest.raises(aiohttp.web.HTTPConflict):
        async_run(link2.add_node(node1, 0, 4))


def test_add_node_cloud(async_run, project, compute):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]
    node2 = Node(project, compute, "node2", node_type="cloud")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()

    async_run(link.add_node(node1, 0, 4))
    async_run(link.add_node(node2, 0, 4))


def test_add_node_cloud_to_cloud(async_run, project, compute):
    """
    Cloud to cloud connection is not allowed
    """
    node1 = Node(project, compute, "node1", node_type="cloud")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]
    node2 = Node(project, compute, "node2", node_type="cloud")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()

    async_run(link.add_node(node1, 0, 4))
    with pytest.raises(aiohttp.web.HTTPConflict):
        async_run(link.add_node(node2, 0, 4))


def test_add_node_same_node(async_run, project, compute):
    """
    Connection to the same node is not allowed
    """
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4), EthernetPort("E1", 0, 0, 5)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()

    async_run(link.add_node(node1, 0, 4))
    with pytest.raises(aiohttp.web.HTTPConflict):
        async_run(link.add_node(node1, 0, 5))


def test_add_node_serial_to_ethernet(async_run, project, compute):
    """
    Serial to ethernet connection is not allowed
    """
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]
    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [SerialPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()

    async_run(link.add_node(node1, 0, 4))
    with pytest.raises(aiohttp.web.HTTPConflict):
        async_run(link.add_node(node2, 0, 4))


def test_json(async_run, project, compute, link):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]
    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 1, 3)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))
    async_run(link.add_node(node2, 1, 3))
    assert link.__json__() == {
        "link_id": link.id,
        "project_id": project.id,
        "nodes": [
            {
                "node_id": node1.id,
                "adapter_number": 0,
                "port_number": 4,
                'label': {
                    'y': -10,
                    'text': '0/4',
                    'x': -10,
                    'rotation': 0,
                    'style': 'font-size: 10; font-style: Verdana'
                }
            },
            {
                "node_id": node2.id,
                "adapter_number": 1,
                "port_number": 3,
                'label': {
                    'y': -10,
                    'text': '1/3',
                    'x': -10,
                    'rotation': 0,
                    'style': 'font-size: 10; font-style: Verdana'
                }
            }
        ],
        "filters": {},
        "suspend": False,
        "link_type": "ethernet",
        "capturing": False,
        "capture_file_name": None,
        "capture_file_path": None
    }
    assert link.__json__(topology_dump=True) == {
        "link_id": link.id,
        "nodes": [
            {
                "node_id": node1.id,
                "adapter_number": 0,
                "port_number": 4,
                'label': {
                    'y': -10,
                    'text': '0/4',
                    'x': -10,
                    'rotation': 0,
                    'style': 'font-size: 10; font-style: Verdana'
                }
            },
            {
                "node_id": node2.id,
                "adapter_number": 1,
                "port_number": 3,
                'label': {
                    'y': -10,
                    'text': '1/3',
                    'x': -10,
                    'rotation': 0,
                    'style': 'font-size: 10; font-style: Verdana'
                }
            }
        ],
        "filters": {},
        "suspend": False
    }


def test_json_serial_link(async_run, project, compute, link):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [SerialPort("S0", 0, 0, 4)]
    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [SerialPort("S0", 0, 1, 3)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))
    async_run(link.add_node(node2, 1, 3))
    assert link.__json__()["link_type"] == "serial"


def test_start_streaming_pcap(link, async_run, tmpdir, project):
    @asyncio.coroutine
    def fake_reader():
        output = AsyncioBytesIO()
        yield from output.write(b"hello")
        output.seek(0)
        return output

    link._capture_file_name = "test.pcap"
    link._capturing = True
    link.read_pcap_from_source = fake_reader
    async_run(link._start_streaming_pcap())
    with open(os.path.join(project.captures_directory, "test.pcap"), "rb") as f:
        c = f.read()
        assert c == b"hello"


def test_default_capture_file_name(project, compute, async_run):
    node1 = Node(project, compute, "Hello@", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]
    node2 = Node(project, compute, "w0.rld", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 1, 3)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))
    async_run(link.add_node(node2, 1, 3))
    assert link.default_capture_file_name() == "Hello_0-4_to_w0rld_1-3.pcap"


def test_start_capture(link, async_run, tmpdir, project, controller):
    @asyncio.coroutine
    def fake_reader():
        return AsyncioBytesIO()

    link.read_pcap_from_source = fake_reader
    controller._notification = MagicMock()
    async_run(link.start_capture(capture_file_name="test.pcap"))
    assert link._capturing
    assert link._capture_file_name == "test.pcap"
    controller._notification.emit.assert_called_with("link.updated", link.__json__())


def test_stop_capture(link, async_run, tmpdir, project, controller):
    link._capturing = True
    controller._notification = MagicMock()
    async_run(link.stop_capture())
    assert link._capturing is False
    controller._notification.emit.assert_called_with("link.updated", link.__json__())


def test_delete(async_run, project, compute):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()
    project.dump = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))

    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]
    async_run(link.add_node(node2, 0, 4))

    assert link in node2.links

    async_run(link.delete())
    assert link not in node2.links


def test_update_filters(async_run, project, compute):
    node1 = Node(project, compute, "node1", node_type="qemu")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    link._project.controller.notification.emit = MagicMock()
    project.dump = AsyncioMagicMock()
    async_run(link.add_node(node1, 0, 4))

    node2 = Node(project, compute, "node2", node_type="qemu")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]
    async_run(link.add_node(node2, 0, 4))

    link.update = AsyncioMagicMock()
    assert link._created
    async_run(link.update_filters({
        "packet_loss": [10],
        "delay": [50, 10],
        "frequency_drop": [0],
        "bpf": [" \n  "]
    }))
    assert link.filters == {
        "packet_loss": [10],
        "delay": [50, 10]
    }
    assert link.update.called


def test_available_filters(async_run, project, compute):
    node1 = Node(project, compute, "node1", node_type="ethernet_switch")
    node1._ports = [EthernetPort("E0", 0, 0, 4)]

    link = Link(project)
    link.create = AsyncioMagicMock()
    assert link.available_filters() == []

    # Ethernet switch is not supported should return 0 filters
    async_run(link.add_node(node1, 0, 4))
    assert link.available_filters() == []

    node2 = Node(project, compute, "node2", node_type="vpcs")
    node2._ports = [EthernetPort("E0", 0, 0, 4)]
    async_run(link.add_node(node2, 0, 4))
    assert len(link.available_filters()) > 0



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import json
import pytest
import socket
import aiohttp
import asyncio
from unittest.mock import patch, MagicMock

from gns3server.controller.project import Project
from gns3server.controller.compute import Compute, ComputeError, ComputeConflict
from gns3server.version import __version__
from tests.utils import asyncio_patch, AsyncioMagicMock


@pytest.fixture
def compute(controller):
    compute = Compute("my_compute_id", protocol="https", host="example.com", port=84, controller=controller)
    compute._connected = True
    return compute


def test_init(compute):
    assert compute.id == "my_compute_id"


def test_getUrl(controller):
    compute = Compute("my_compute_id", protocol="https", host="localhost", port=84, controller=controller)
    assert compute._getUrl("/test") == "https://localhost:84/v2/compute/test"
    # IPV6 localhost
    compute = Compute("my_compute_id", protocol="https", host="::1", port=84, controller=controller)
    assert compute._getUrl("/test") == "https://[::1]:84/v2/compute/test"

    # Listen on all interfaces aka 0.0.0.0 require us to connect via 127.0.0.1
    compute = Compute("my_compute_id", protocol="https", host="0.0.0.0", port=84, controller=controller)
    assert compute._getUrl("/test") == "https://127.0.0.1:84/v2/compute/test"
    # IPV6
    compute = Compute("my_compute_id", protocol="https", host="::", port=84, controller=controller)
    assert compute._getUrl("/test") == "https://[::1]:84/v2/compute/test"


def test_get_url(controller):
    compute = Compute("my_compute_id", protocol="https", host="localhost", port=84, controller=controller)
    with patch('gns3server.controller.compute.Compute._getUrl', return_value="returned") as getURL:
        assert compute.get_url("/test") == 'returned'
        getURL.assert_called_once_with('/test')


def test_host_ip(controller):
    compute = Compute("my_compute_id", protocol="https", host="localhost", port=84, controller=controller)
    assert compute.host_ip == "127.0.0.1"


def test_name():
    c = Compute("my_compute_id", protocol="https", host="example.com", port=84, controller=MagicMock(), name=None)
    assert c.name == "https://example.com:84"
    c = Compute("world", protocol="https", host="example.com", port=84, controller=MagicMock(), name="hello")
    assert c.name == "hello"
    c = Compute("world", protocol="https", host="example.com", port=84, controller=MagicMock(), user="azertyuiopqsdfghjklkm")
    assert c.name == "https://azertyuiopq...@example.com:84"


def test_compute_httpQuery(compute, async_run):
    response = MagicMock()
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        response.status = 200

        async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_called_with("POST", "https://example.com:84/v2/compute/projects", data=b'{"a": "b"}', headers={'content-type': 'application/json'}, auth=None, chunked=None, timeout=20)
        assert compute._auth is None


def test_compute_httpQueryAuth(compute, async_run):
    response = MagicMock()
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        response.status = 200

        compute.user = "root"
        compute.password = "toor"
        async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_called_with("POST", "https://example.com:84/v2/compute/projects", data=b'{"a": "b"}', headers={'content-type': 'application/json'}, auth=compute._auth, chunked=None, timeout=20)
        assert compute._auth.login == "root"
        assert compute._auth.password == "toor"


def test_compute_httpQueryNotConnected(compute, controller, async_run):
    controller._notification = MagicMock()
    compute._connected = False
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=json.dumps({"version": __version__}).encode())
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/capabilities", headers={'content-type': 'application/json'}, data=None, auth=None, chunked=None, timeout=20)
        mock.assert_any_call("POST", "https://example.com:84/v2/compute/projects", data=b'{"a": "b"}', headers={'content-type': 'application/json'}, auth=None, chunked=None, timeout=20)
    assert compute._connected
    assert compute._capabilities["version"] == __version__
    controller.notification.emit.assert_called_with("compute.updated", compute.__json__())


def test_compute_httpQueryNotConnectedGNS3vmNotRunning(compute, controller, async_run):
    """
    We are not connected to the remote and it's a GNS3 VM. So we need to start it
    """
    controller._notification = MagicMock()
    controller.gns3vm = AsyncioMagicMock()
    controller.gns3vm.running = False

    compute._id = "vm"
    compute._connected = False
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=json.dumps({"version": __version__}).encode())
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/capabilities", headers={'content-type': 'application/json'}, data=None, auth=None, chunked=None, timeout=20)
        mock.assert_any_call("POST", "https://example.com:84/v2/compute/projects", data=b'{"a": "b"}', headers={'content-type': 'application/json'}, auth=None, chunked=None, timeout=20)

    assert controller.gns3vm.start.called
    assert compute._connected
    assert compute._capabilities["version"] == __version__
    controller.notification.emit.assert_called_with("compute.updated", compute.__json__())


def test_compute_httpQueryNotConnectedInvalidVersion(compute, async_run):
    compute._connected = False
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=json.dumps({"version": "1.42.4"}).encode())
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        with pytest.raises(aiohttp.web.HTTPConflict):
            async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/capabilities", headers={'content-type': 'application/json'}, data=None, auth=None, chunked=None, timeout=20)


def test_compute_httpQueryNotConnectedNonGNS3Server(compute, async_run):
    compute._connected = False
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=b'Blocked by super antivirus')
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        with pytest.raises(aiohttp.web.HTTPConflict):
            async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/capabilities", headers={'content-type': 'application/json'}, data=None, auth=None, chunked=None, timeout=20)


def test_compute_httpQueryNotConnectedNonGNS3Server2(compute, async_run):
    compute._connected = False
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=b'{}')
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        with pytest.raises(aiohttp.web.HTTPConflict):
            async_run(compute.post("/projects", {"a": "b"}))
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/capabilities", headers={'content-type': 'application/json'}, data=None, auth=None, chunked=None, timeout=20)


def test_compute_httpQueryError(compute, async_run):
    response = MagicMock()
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        response.status = 404

        with pytest.raises(aiohttp.web.HTTPNotFound):
            async_run(compute.post("/projects", {"a": "b"}))


def test_compute_httpQueryConflictError(compute, async_run):
    response = MagicMock()
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        response.status = 409
        response.read = AsyncioMagicMock(return_value=b'{"message": "Test"}')

        with pytest.raises(ComputeConflict):
            async_run(compute.post("/projects", {"a": "b"}))


def test_compute_httpQuery_project(compute, async_run):
    response = MagicMock()
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        response.status = 200

        project = Project(name="Test")
        async_run(compute.post("/projects", project))
        mock.assert_called_with("POST", "https://example.com:84/v2/compute/projects", data=json.dumps(project.__json__()), headers={'content-type': 'application/json'}, auth=None, chunked=None, timeout=20)


def test_connectNotification(compute, async_run):
    ws_mock = AsyncioMagicMock()

    call = 0

    @asyncio.coroutine
    def receive():
        nonlocal call
        call += 1
        if call == 1:
            response = MagicMock()
            response.data = '{"action": "test", "event": {"a": 1}}'
            response.tp = aiohttp.WSMsgType.text
            return response
        else:
            response = MagicMock()
            response.tp = aiohttp.WSMsgType.closed
            return response

    compute._controller._notification = MagicMock()
    compute._http_session = AsyncioMagicMock(return_value=ws_mock)
    compute._http_session.ws_connect = AsyncioMagicMock(return_value=ws_mock)
    ws_mock.receive = receive
    async_run(compute._connect_notification())

    compute._controller.notification.dispatch.assert_called_with('test', {'a': 1}, compute_id=compute.id)
    assert compute._connected is False


def test_connectNotificationPing(compute, async_run):
    """
    When we receive a ping from a compute we update
    the compute memory and CPU usage
    """
    ws_mock = AsyncioMagicMock()

    call = 0

    @asyncio.coroutine
    def receive():
        nonlocal call
        call += 1
        if call == 1:
            response = MagicMock()
            response.data = '{"action": "ping", "event": {"cpu_usage_percent": 35.7, "memory_usage_percent": 80.7}}'
            response.tp = aiohttp.WSMsgType.text
            return response
        else:
            response = MagicMock()
            response.tp = aiohttp.WSMsgType.closed
            return response

    compute._controller._notification = MagicMock()
    compute._http_session = AsyncioMagicMock(return_value=ws_mock)
    compute._http_session.ws_connect = AsyncioMagicMock(return_value=ws_mock)
    ws_mock.receive = receive
    async_run(compute._connect_notification())

    assert not compute._controller.notification.dispatch.called
    args, _ = compute._controller.notification.emit.call_args_list[0]
    assert args[0] == "compute.updated"
    assert args[1]["memory_usage_percent"] == 80.7
    assert args[1]["cpu_usage_percent"] == 35.7


def test_json(compute):
    compute.user = "test"
    assert compute.__json__() == {
        "compute_id": "my_compute_id",
        "name": compute.name,
        "protocol": "https",
        "host": "example.com",
        "port": 84,
        "user": "test",
        "cpu_usage_percent": None,
        "memory_usage_percent": None,
        "connected": True,
        "last_error": None,
        "capabilities": {
            "version": None,
            "node_types": []
        }
    }
    assert compute.__json__(topology_dump=True) == {
        "compute_id": "my_compute_id",
        "name": compute.name,
        "protocol": "https",
        "host": "example.com",
        "port": 84,
    }


def test_streamFile(project, async_run, compute):
    response = MagicMock()
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        async_run(compute.stream_file(project, "test/titi", timeout=120))
    mock.assert_called_with("GET", "https://example.com:84/v2/compute/projects/{}/stream/test/titi".format(project.id), auth=None, timeout=120)


def test_downloadFile(project, async_run, compute):
    response = MagicMock()
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        async_run(compute.download_file(project, "test/titi"))
    mock.assert_called_with("GET", "https://example.com:84/v2/compute/projects/{}/files/test/titi".format(project.id), auth=None)


def test_close(compute, async_run):
    assert compute.connected is True
    async_run(compute.close())
    assert compute.connected is False


def test_update(compute, controller, async_run):
    compute._controller._notification = MagicMock()
    compute._controller.save = MagicMock()
    compute.name = "Test"
    compute.host = "example.org"
    compute._connected = True
    async_run(compute.update(name="Test 2"))
    assert compute.name == "Test 2"
    assert compute.host == "example.org"
    controller.notification.emit.assert_called_with("compute.updated", compute.__json__())
    assert compute.connected is False
    assert compute._controller.save.called


def test_forward_get(compute, async_run):
    response = MagicMock()
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        async_run(compute.forward("GET", "qemu", "images"))
        mock.assert_called_with("GET", "https://example.com:84/v2/compute/qemu/images", auth=None, data=None, headers={'content-type': 'application/json'}, chunked=None, timeout=None)


def test_forward_404(compute, async_run):
    response = MagicMock()
    response.status = 404
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        with pytest.raises(aiohttp.web_exceptions.HTTPNotFound):
            async_run(compute.forward("GET", "qemu", "images"))


def test_forward_post(compute, async_run):
    response = MagicMock()
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        async_run(compute.forward("POST", "qemu", "img", data={"id": 42}))
        mock.assert_called_with("POST", "https://example.com:84/v2/compute/qemu/img", auth=None, data=b'{"id": 42}', headers={'content-type': 'application/json'}, chunked=None, timeout=None)


def test_images(compute, async_run, images_dir):
    """
    Will return image on compute and on controller
    """
    response = MagicMock()
    response.status = 200
    response.read = AsyncioMagicMock(return_value=json.dumps([{
        "filename": "linux.qcow2",
        "path": "linux.qcow2",
        "md5sum": "d41d8cd98f00b204e9800998ecf8427e",
        "filesize": 0}]).encode())
    open(os.path.join(images_dir, "QEMU", "asa.qcow2"), "w+").close()
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        images = async_run(compute.images("qemu"))
        mock.assert_called_with("GET", "https://example.com:84/v2/compute/qemu/images", auth=None, data=None, headers={'content-type': 'application/json'}, chunked=None, timeout=None)

    assert images == [
        {"filename": "asa.qcow2", "path": "asa.qcow2", "md5sum": "d41d8cd98f00b204e9800998ecf8427e", "filesize": 0},
        {"filename": "linux.qcow2", "path": "linux.qcow2", "md5sum": "d41d8cd98f00b204e9800998ecf8427e", "filesize": 0}
    ]


def test_list_files(project, async_run, compute):
    res = [{"path": "test"}]
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=json.dumps(res).encode())
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        assert async_run(compute.list_files(project)) == res
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/projects/{}/files".format(project.id), auth=None, chunked=None, data=None, headers={'content-type': 'application/json'}, timeout=None)


def test_interfaces(project, async_run, compute):
    res = [
        {
            "id": "vmnet99",
            "ip_address": "172.16.97.1",
            "mac_address": "00:50:56:c0:00:63",
            "name": "vmnet99",
            "netmask": "255.255.255.0",
            "type": "ethernet"
        }
    ]
    response = AsyncioMagicMock()
    response.read = AsyncioMagicMock(return_value=json.dumps(res).encode())
    response.status = 200
    with asyncio_patch("aiohttp.ClientSession.request", return_value=response) as mock:
        assert async_run(compute.interfaces()) == res
        mock.assert_any_call("GET", "https://example.com:84/v2/compute/network/interfaces", auth=None, chunked=None, data=None, headers={'content-type': 'application/json'}, timeout=20)


def test_get_ip_on_same_subnet(controller, async_run):
    compute1 = Compute("compute1", host="192.168.1.1", controller=controller)
    compute1._interfaces_cache = [
        {
            "ip_address": "127.0.0.1",
            "netmask": "255.255.255.255"
        },
        {
            "ip_address": "192.168.2.1",
            "netmask": "255.255.255.0"
        },
        {
            "ip_address": "192.168.1.1",
            "netmask": "255.255.255.0"
        },
    ]

    # Case 1 both host are on the same network
    compute2 = Compute("compute2", host="192.168.1.2", controller=controller)
    compute2._interfaces_cache = [
        {
            "ip_address": "127.0.0.1",
            "netmask": "255.255.255.255"
        },
        {
            "ip_address": "192.168.2.2",
            "netmask": "255.255.255.0"
        },
        {
            "ip_address": "192.168.1.2",
            "netmask": "255.255.255.0"
        }
    ]
    assert async_run(compute1.get_ip_on_same_subnet(compute2)) == ("192.168.1.1", "192.168.1.2")

    # Case 2 compute2 host is on a different network but a common interface is available
    compute2 = Compute("compute2", host="127.0.0.1", controller=controller)
    compute2._interfaces_cache = [
        {
            "ip_address": "127.0.0.1",
            "netmask": "255.255.255.255"
        },
        {
            "ip_address": "192.168.4.2",
            "netmask": "255.255.255.0"
        },
        {
            "ip_address": "192.168.1.2",
            "netmask": "255.255.255.0"
        }
    ]
    assert async_run(compute1.get_ip_on_same_subnet(compute2)) == ("192.168.1.1", "192.168.1.2")

    #No common interface
    compute2 = Compute("compute2", host="127.0.0.1", controller=controller)
    compute2._interfaces_cache = [
        {
            "ip_address": "127.0.0.1",
            "netmask": "255.255.255.255"
        }
    ]
    with pytest.raises(ValueError):
        async_run(compute1.get_ip_on_same_subnet(compute2))

    # Ignore 169.254 network because it's for Windows special purpose
    compute2 = Compute("compute2", host="192.168.1.2", controller=controller)
    compute1 = Compute("compute1", host="192.168.2.1", controller=controller)
    compute1._interfaces_cache = [
        {
            "ip_address": "127.0.0.1",
            "netmask": "255.255.255.255"
        },
        {
            "ip_address": "169.254.1.1",
            "netmask": "255.255.0.0"
        },
    ]
    compute2._interfaces_cache = [
        {
            "ip_address": "127.0.0.1",
            "netmask": "255.255.255.255"
        },
        {
            "ip_address": "169.254.2.1",
            "netmask": "255.255.0.0"
        },
    ]
    assert async_run(compute1.get_ip_on_same_subnet(compute2)) == ('192.168.2.1', '192.168.1.2')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3server.web.route import Route
from gns3server.controller import Controller

from gns3server.schemas.drawing import (
    DRAWING_OBJECT_SCHEMA,
)


class DrawingHandler:
    """
    API entry point for Drawing
    """

    @Route.get(
        r"/projects/{project_id}/drawings",
        parameters={
            "project_id": "Project UUID"
        },
        status_codes={
            200: "List of drawings returned",
        },
        description="List drawings of a project")
    def list_drawings(request, response):

        project = yield from Controller.instance().get_loaded_project(request.match_info["project_id"])
        response.json([v for v in project.drawings.values()])

    @Route.post(
        r"/projects/{project_id}/drawings",
        parameters={
            "project_id": "Project UUID"
        },
        status_codes={
            201: "Drawing created",
            400: "Invalid request"
        },
        description="Create a new drawing instance",
        input=DRAWING_OBJECT_SCHEMA,
        output=DRAWING_OBJECT_SCHEMA)
    def create(request, response):

        project = yield from Controller.instance().get_loaded_project(request.match_info["project_id"])
        drawing = yield from project.add_drawing(**request.json)
        response.set_status(201)
        response.json(drawing)

    @Route.get(
        r"/projects/{project_id}/drawings/{drawing_id}",
        parameters={
            "project_id": "Project UUID",
            "drawing_id": "Drawing UUID"
        },
        status_codes={
            200: "Drawing found",
            400: "Invalid request",
            404: "Drawing doesn't exist"
        },
        description="Get a drawing instance",
        output=DRAWING_OBJECT_SCHEMA)
    def get_drawing(request, response):

        project = yield from Controller.instance().get_loaded_project(request.match_info["project_id"])
        drawing = project.get_drawing(request.match_info["drawing_id"])
        response.set_status(200)
        response.json(drawing)

    @Route.put(
        r"/projects/{project_id}/drawings/{drawing_id}",
        parameters={
            "project_id": "Project UUID",
            "drawing_id": "Drawing UUID"
        },
        status_codes={
            201: "Drawing updated",
            400: "Invalid request"
        },
        description="Update a drawing instance",
        input=DRAWING_OBJECT_SCHEMA,
        output=DRAWING_OBJECT_SCHEMA)
    def update(request, response):

        project = yield from Controller.instance().get_loaded_project(request.match_info["project_id"])
        drawing = project.get_drawing(request.match_info["drawing_id"])
        yield from drawing.update(**request.json)
        response.set_status(201)
        response.json(drawing)

    @Route.delete(
        r"/projects/{project_id}/drawings/{drawing_id}",
        parameters={
            "project_id": "Project UUID",
            "drawing_id": "Drawing UUID"
        },
        status_codes={
            204: "Drawing deleted",
            400: "Invalid request"
        },
        description="Delete a drawing instance")
    def delete(request, response):

        project = yield from Controller.instance().get_loaded_project(request.match_info["project_id"])
        yield from project.delete_drawing(request.match_info["drawing_id"])
        response.set_status(204)



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import uuid
import shutil
import tempfile
import asyncio
import aiohttp.web
from datetime import datetime, timezone

from ..utils.asyncio import wait_run_in_executor
from .export_project import export_project
from .import_project import import_project


# The string use to extract the date from the filename
FILENAME_TIME_FORMAT = "%d%m%y_%H%M%S"


class Snapshot:
    """
    A snapshot object
    """

    def __init__(self, project, name=None, filename=None):

        assert filename or name, "You need to pass a name or a filename"

        self._id = str(uuid.uuid4())  # We don't need to keep id between project loading because they are use only as key for operation like delete, update.. but have no impact on disk
        self._project = project
        if name:
            self._name = name
            self._created_at = datetime.now().timestamp()
            filename = self._name + "_" + datetime.utcfromtimestamp(self._created_at).replace(tzinfo=None).strftime(FILENAME_TIME_FORMAT) + ".gns3project"
        else:
            self._name = filename.split("_")[0]
            datestring = filename.replace(self._name + "_", "").split(".")[0]
            try:
                self._created_at = datetime.strptime(datestring, FILENAME_TIME_FORMAT).replace(tzinfo=timezone.utc).timestamp()
            except ValueError:
                self._created_at = datetime.utcnow().timestamp()
        self._path = os.path.join(project.path, "snapshots", filename)

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._name

    @property
    def path(self):
        return self._path

    @property
    def created_at(self):
        return int(self._created_at)

    def _create_snapshot_file(self, zipstream):
        """
        Creates the snapshot file (to be run in its own thread)
        """

        with open(self.path, "wb") as f:
            for data in zipstream:
                f.write(data)

    @asyncio.coroutine
    def create(self):
        """
        Create the snapshot
        """

        if os.path.exists(self.path):
            raise aiohttp.web.HTTPConflict(text="The snapshot file '{}' already exists".format(self.name))

        snapshot_directory = os.path.join(self._project.path, "snapshots")
        try:
            os.makedirs(snapshot_directory, exist_ok=True)
        except OSError as e:
            raise aiohttp.web.HTTPInternalServerError(text="Could not create the snapshot directory '{}': {}".format(snapshot_directory, e))

        try:
            with tempfile.TemporaryDirectory() as tmpdir:
                zipstream = yield from export_project(self._project, tmpdir, keep_compute_id=True, allow_all_nodes=True)
                yield from wait_run_in_executor(self._create_snapshot_file, zipstream)
        except (ValueError, OSError, RuntimeError) as e:
            raise aiohttp.web.HTTPConflict(text="Could not create snapshot file '{}': {}".format(self.path, e))

    @asyncio.coroutine
    def restore(self):
        """
        Restore the snapshot
        """

        yield from self._project.delete_on_computes()
        # We don't send close notification to clients because the close / open dance is purely internal
        yield from self._project.close(ignore_notification=True)

        try:
            # delete the current project files
            project_files_path = os.path.join(self._project.path, "project-files")
            if os.path.exists(project_files_path):
                yield from wait_run_in_executor(shutil.rmtree, project_files_path)
            with open(self._path, "rb") as f:
                project = yield from import_project(self._project.controller, self._project.id, f, location=self._project.path)
        except (OSError, PermissionError) as e:
            raise aiohttp.web.HTTPConflict(text=str(e))
        yield from project.open()
        self._project.controller.notification.emit("snapshot.restored", self.__json__())
        return self._project

    def __json__(self):
        return {
            "snapshot_id": self._id,
            "name": self._name,
            "created_at": int(self._created_at),
            "project_id": self._project.id
        }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from ..error import NodeError
from .nodes.cloud import Cloud
from .nodes.nat import Nat
from .nodes.ethernet_hub import EthernetHub
from .nodes.ethernet_switch import EthernetSwitch

import logging
log = logging.getLogger(__name__)

BUILTIN_NODES = {'cloud': Cloud,
                 'nat': Nat,
                 'ethernet_hub': EthernetHub,
                 'ethernet_switch': EthernetSwitch}


class BuiltinNodeFactory:

    """
    Factory to create an builtin object based on the node type.
    """

    def __new__(cls, name, node_id, project, manager, node_type, **kwargs):

        if node_type not in BUILTIN_NODES:
            raise NodeError("Unknown node type: {}".format(node_type))

        return BUILTIN_NODES[node_type](name, node_id, project, manager, **kwargs)



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

from .cloud import Cloud
from ...error import NodeError

import gns3server.utils.interfaces


class Nat(Cloud):
    """
    A portable and preconfigured node allowing topology to get a
    nat access to the outside
    """

    def __init__(self, *args, **kwargs):

        if sys.platform.startswith("linux"):
            if "virbr0" not in [interface["name"] for interface in gns3server.utils.interfaces.interfaces()]:
                raise NodeError("virbr0 is missing. You need to install libvirt")
            interface = "virbr0"
        else:
            interfaces = list(filter(lambda x: 'vmnet8' in x.lower(),
                           [interface["name"] for interface in gns3server.utils.interfaces.interfaces()]))
            if not len(interfaces):
                raise NodeError("vmnet8 is missing. You need to install VMware or use the NAT node on GNS3 VM")
            interface = interfaces[0]  # take the first available interface containing the vmnet8 name

        ports = [
            {
                "name": "nat0",
                "type": "ethernet",
                "interface": interface,
                "port_number": 0
            }
        ]
        super().__init__(*args, ports=ports)

    @property
    def ports_mapping(self):
        return self._ports_mapping

    @ports_mapping.setter
    def ports_mapping(self, ports):
        # It's not allowed to change it
        pass

    @classmethod
    def is_supported(self):
        return True

    def __json__(self):
        return {
            "name": self.name,
            "node_id": self.id,
            "project_id": self.project.id,
            "status": "started",
            "ports_mapping": self.ports_mapping
        }



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import uuid
import pytest
import aiohttp
from unittest.mock import MagicMock
from tests.utils import asyncio_patch

from gns3server.controller.project import Project
from gns3server.controller.compute import Compute
from gns3server.controller.topology import project_to_topology, load_topology, GNS3_FILE_FORMAT_REVISION
from gns3server.version import __version__


def test_project_to_topology_empty(tmpdir):
    project = Project(name="Test")
    topo = project_to_topology(project)
    assert topo == {
        "project_id": project.id,
        "name": "Test",
        "auto_start": False,
        "auto_close": True,
        "auto_open": False,
        "scene_width": 2000,
        "scene_height": 1000,
        "revision": GNS3_FILE_FORMAT_REVISION,
        "zoom": 100,
        "show_grid": False,
        "show_interface_labels": False,
        "show_layers": False,
        "snap_to_grid": False,
        "grid_size": 0,
        "topology": {
            "nodes": [],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "type": "topology",
        "supplier": None,
        "variables": None,
        "version": __version__
    }


def test_basic_topology(tmpdir, async_run, controller):
    project = Project(name="Test", controller=controller)
    compute = Compute("my_compute", controller)
    compute.http_query = MagicMock()

    with asyncio_patch("gns3server.controller.node.Node.create"):
        node1 = async_run(project.add_node(compute, "Node 1", str(uuid.uuid4()), node_type="qemu"))
        node2 = async_run(project.add_node(compute, "Node 2", str(uuid.uuid4()), node_type="qemu"))

    link = async_run(project.add_link())
    async_run(link.add_node(node1, 0, 0))
    with asyncio_patch("gns3server.controller.udp_link.UDPLink.create") as mock_udp_create:
        async_run(link.add_node(node2, 0, 0))

    drawing = async_run(project.add_drawing(svg="<svg></svg>"))

    topo = project_to_topology(project)
    assert len(topo["topology"]["nodes"]) == 2
    assert node1.__json__(topology_dump=True) in topo["topology"]["nodes"]
    assert topo["topology"]["links"][0] == link.__json__(topology_dump=True)
    assert topo["topology"]["computes"][0] == compute.__json__(topology_dump=True)
    assert topo["topology"]["drawings"][0] == drawing.__json__(topology_dump=True)


def test_project_to_topology(tmpdir, controller):
    variables = [
        {"name": "TEST1"},
        {"name": "TEST2", "value": "value1"}
    ]
    supplier = {
        'logo': 'logo.png',
        'url': 'http://example.com'
    }

    project = Project(name="Test", controller=controller)
    compute = Compute("my_compute", controller)
    compute.http_query = MagicMock()
    project.variables = variables
    project.supplier = supplier
    topo = project_to_topology(project)
    assert topo["variables"] == variables
    assert topo["supplier"] == supplier


def test_load_topology(tmpdir):
    data = {
        "project_id": "69f26504-7aa3-48aa-9f29-798d44841211",
        "name": "Test",
        "revision": GNS3_FILE_FORMAT_REVISION,
        "topology": {
            "nodes": [],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "type": "topology",
        "version": __version__}

    path = str(tmpdir / "test.gns3")
    with open(path, "w+") as f:
        json.dump(data, f)
    topo = load_topology(path)
    assert topo == data


def test_load_topology_file_error(tmpdir):
    path = str(tmpdir / "test.gns3")
    with pytest.raises(aiohttp.web.HTTPConflict):
        topo = load_topology(path)


def test_load_topology_file_error_schema_error(tmpdir):
    path = str(tmpdir / "test.gns3")
    with open(path, "w+") as f:
        json.dump({
            "revision": GNS3_FILE_FORMAT_REVISION
        }, f)
    with pytest.raises(aiohttp.web.HTTPConflict):
        topo = load_topology(path)


def test_load_newer_topology(tmpdir):
    """
    If a topology is design for a more recent GNS3 version
    we disallow the loading.
    """
    data = {
        "project_id": "69f26504-7aa3-48aa-9f29-798d44841211",
        "name": "Test",
        "revision": 42,
        "topology": {
        },
        "type": "topology",
        "version": __version__}

    path = str(tmpdir / "test.gns3")
    with open(path, "w+") as f:
        json.dump(data, f)
    with pytest.raises(aiohttp.web.HTTPConflict):
        topo = load_topology(path)


def test_load_topology_with_variables(tmpdir):
    variables = [
        {"name": "TEST1"},
        {"name": "TEST2", "value": "value1"}
    ]
    data = {
        "project_id": "69f26504-7aa3-48aa-9f29-798d44841211",
        "name": "Test",
        "revision": GNS3_FILE_FORMAT_REVISION,
        "topology": {
            "nodes": [],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "variables": variables,
        "type": "topology",
        "version": __version__}

    path = str(tmpdir / "test.gns3")
    with open(path, "w+") as f:
        json.dump(data, f)
    topo = load_topology(path)
    assert topo == data


def test_load_topology_with_supplier(tmpdir):
    supplier = {
        'logo': 'logo.png',
        'url': 'http://example.com'
    }
    data = {
        "project_id": "69f26504-7aa3-48aa-9f29-798d44841211",
        "name": "Test",
        "revision": GNS3_FILE_FORMAT_REVISION,
        "topology": {
            "nodes": [],
            "links": [],
            "computes": [],
            "drawings": []
        },
        "supplier": supplier,
        "type": "topology",
        "version": __version__}

    path = str(tmpdir / "test.gns3")
    with open(path, "w+") as f:
        json.dump(data, f)
    topo = load_topology(path)
    assert topo == data


#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import uuid
import json
import pytest
import socket
import aiohttp
from unittest.mock import MagicMock, patch
from tests.utils import AsyncioMagicMock, asyncio_patch

from gns3server.controller.compute import Compute
from gns3server.version import __version__


def test_save(controller, controller_config_path):
    controller.save()
    assert os.path.exists(controller_config_path)
    with open(controller_config_path) as f:
        data = json.load(f)
        assert data["computes"] == []
        assert data["version"] == __version__
        assert data["settings"] == {}
        assert data["gns3vm"] == controller.gns3vm.__json__()


def test_load_controller_settings(controller, controller_config_path, async_run):
    controller.save()
    with open(controller_config_path) as f:
        data = json.load(f)
    data["computes"] = [
        {
            "host": "localhost",
            "port": 8000,
            "protocol": "http",
            "user": "admin",
            "password": "root",
            "compute_id": "test1"
        }
    ]
    data["settings"] = {"IOU": {"test": True}}
    data["gns3vm"] = {"vmname": "Test VM"}
    with open(controller_config_path, "w+") as f:
        json.dump(data, f)
    assert len(async_run(controller._load_controller_settings())) == 1
    assert controller.settings["IOU"]
    assert controller.gns3vm.settings["vmname"] == "Test VM"


def test_load_controller_settings_with_no_computes_section(controller, controller_config_path, async_run):
    controller.save()
    with open(controller_config_path) as f:
        data = json.load(f)
    del data['computes']
    with open(controller_config_path, "w+") as f:
        json.dump(data, f)
    assert len(async_run(controller._load_controller_settings())) == 0


def test_import_computes_1_x(controller, controller_config_path, async_run):
    """
    At first start the server should import the
    computes from the gns3_gui 1.X
    """
    gns3_gui_conf = {
        "Servers": {
            "remote_servers": [
                {
                    "host": "127.0.0.1",
                    "password": "",
                    "port": 3081,
                    "protocol": "http",
                    "url": "http://127.0.0.1:3081",
                    "user": ""
                }
            ]
        }
    }
    config_dir = os.path.dirname(controller_config_path)
    os.makedirs(config_dir, exist_ok=True)
    with open(os.path.join(config_dir, "gns3_gui.conf"), "w+") as f:
        json.dump(gns3_gui_conf, f)

    async_run(controller._load_controller_settings())
    for compute in controller.computes.values():
        if compute.id != "local":
            assert len(compute.id) == 36
            assert compute.host == "127.0.0.1"
            assert compute.port == 3081
            assert compute.protocol == "http"
            assert compute.name == "http://127.0.0.1:3081"
            assert compute.user is None
            assert compute.password is None


def test_import_gns3vm_1_x(controller, controller_config_path, async_run):
    """
    At first start the server should import the
    gns3vm settings from the gns3_gui 1.X
    """
    gns3_gui_conf = {
        "Servers": {
            "vm": {
                "adjust_local_server_ip": True,
                "auto_start": True,
                "auto_stop": False,
                "headless": True,
                "remote_vm_host": "",
                "remote_vm_password": "",
                "remote_vm_port": 3080,
                "remote_vm_protocol": "http",
                "remote_vm_url": "",
                "remote_vm_user": "",
                "virtualization": "VMware",
                "vmname": "GNS3 VM",
                "vmx_path": "/Users/joe/Documents/Virtual Machines.localized/GNS3 VM.vmwarevm/GNS3 VM.vmx"
            }
        }
    }
    config_dir = os.path.dirname(controller_config_path)
    os.makedirs(config_dir, exist_ok=True)
    with open(os.path.join(config_dir, "gns3_gui.conf"), "w+") as f:
        json.dump(gns3_gui_conf, f)

    controller.gns3vm.settings["engine"] = None
    async_run(controller._load_controller_settings())
    assert controller.gns3vm.settings["engine"] == "vmware"
    assert controller.gns3vm.settings["enable"]
    assert controller.gns3vm.settings["headless"]
    assert controller.gns3vm.settings["when_exit"] == "keep"
    assert controller.gns3vm.settings["vmname"] == "GNS3 VM"


def test_import_remote_gns3vm_1_x(controller, controller_config_path, async_run):
    """
    At first start the server should import the
    computes and remote GNS3 VM from the gns3_gui 1.X
    """
    gns3_gui_conf = {
        "Servers": {
            "remote_servers": [
                {
                    "host": "127.0.0.1",
                    "password": "",
                    "port": 3080,
                    "protocol": "http",
                    "url": "http://127.0.0.1:3080",
                    "user": ""
                },
                {
                    "host": "127.0.0.1",
                    "password": "",
                    "port": 3081,
                    "protocol": "http",
                    "url": "http://127.0.0.1:3081",
                    "user": ""
                }
            ],
            "vm": {
                "adjust_local_server_ip": True,
                "auto_start": True,
                "auto_stop": False,
                "headless": True,
                "remote_vm_host": "127.0.0.1",
                "remote_vm_password": "",
                "remote_vm_port": 3081,
                "remote_vm_protocol": "http",
                "remote_vm_url": "http://127.0.0.1:3081",
                "remote_vm_user": "",
                "virtualization": "remote",
                "vmname": "GNS3 VM",
                "vmx_path": "/Users/joe/Documents/Virtual Machines.localized/GNS3 VM.vmwarevm/GNS3 VM.vmx"
            }
        }
    }
    config_dir = os.path.dirname(controller_config_path)
    os.makedirs(config_dir, exist_ok=True)
    with open(os.path.join(config_dir, "gns3_gui.conf"), "w+") as f:
        json.dump(gns3_gui_conf, f)

    with asyncio_patch("gns3server.controller.compute.Compute.connect"):
        async_run(controller._load_controller_settings())
    assert controller.gns3vm.settings["engine"] == "remote"
    assert controller.gns3vm.settings["vmname"] == "http://127.0.0.1:3081"


def test_settings(controller):
    controller._notification = MagicMock()
    controller.settings = {"a": 1}
    controller._notification.emit.assert_called_with("settings.updated", controller.settings)
    assert controller.settings["modification_uuid"] is not None


def test_load_projects(controller, projects_dir, async_run):
    controller.save()

    os.makedirs(os.path.join(projects_dir, "project1"))
    with open(os.path.join(projects_dir, "project1", "project1.gns3"), "w+") as f:
        f.write("")
    with asyncio_patch("gns3server.controller.Controller.load_project") as mock_load_project:
        async_run(controller.load_projects())
    mock_load_project.assert_called_with(os.path.join(projects_dir, "project1", "project1.gns3"), load=False)


def test_add_compute(controller, controller_config_path, async_run):
    controller._notification = MagicMock()
    c = async_run(controller.add_compute(compute_id="test1", connect=False))
    controller._notification.emit.assert_called_with("compute.created", c.__json__())
    assert len(controller.computes) == 1
    async_run(controller.add_compute(compute_id="test1", connect=False))
    controller._notification.emit.assert_called_with("compute.updated", c.__json__())
    assert len(controller.computes) == 1
    async_run(controller.add_compute(compute_id="test2", connect=False))
    assert len(controller.computes) == 2


def test_addDuplicateCompute(controller, controller_config_path, async_run):
    controller._notification = MagicMock()
    c = async_run(controller.add_compute(compute_id="test1", name="Test", connect=False))
    assert len(controller.computes) == 1
    with pytest.raises(aiohttp.web.HTTPConflict):
        async_run(controller.add_compute(compute_id="test2", name="Test", connect=False))


def test_deleteCompute(controller, controller_config_path, async_run):
    c = async_run(controller.add_compute(compute_id="test1", connect=False))
    assert len(controller.computes) == 1
    controller._notification = MagicMock()
    c._connected = True
    async_run(controller.delete_compute("test1"))
    assert len(controller.computes) == 0
    controller._notification.emit.assert_called_with("compute.deleted", c.__json__())
    with open(controller_config_path) as f:
        data = json.load(f)
        assert len(data["computes"]) == 0
    assert c.connected is False


def test_deleteComputeProjectOpened(controller, controller_config_path, async_run):
    """
    When you delete a compute the project using it are close
    """
    c = async_run(controller.add_compute(compute_id="test1", connect=False))
    c.post = AsyncioMagicMock()
    assert len(controller.computes) == 1

    project1 = async_run(controller.add_project(name="Test1"))
    async_run(project1.open())
    # We simulate that the project use this compute
    project1._project_created_on_compute.add(c)

    project2 = async_run(controller.add_project(name="Test2"))
    async_run(project2.open())

    controller._notification = MagicMock()
    c._connected = True
    async_run(controller.delete_compute("test1"))
    assert len(controller.computes) == 0
    controller._notification.emit.assert_called_with("compute.deleted", c.__json__())
    with open(controller_config_path) as f:
        data = json.load(f)
        assert len(data["computes"]) == 0
    assert c.connected is False

    # Project 1 use this compute it should be close before deleting the compute
    assert project1.status == "closed"
    assert project2.status == "opened"


def test_addComputeConfigFile(controller, controller_config_path, async_run):
    async_run(controller.add_compute(compute_id="test1", name="Test", connect=False))
    assert len(controller.computes) == 1
    with open(controller_config_path) as f:
        data = json.load(f)
        assert data["computes"] == [
            {
                'compute_id': 'test1',
                'name': 'Test',
                'host': 'localhost',
                'port': 3080,
                'protocol': 'http',
                'user': None,
                'password': None
            }
        ]


def test_getCompute(controller, async_run):
    compute = async_run(controller.add_compute(compute_id="test1", connect=False))

    assert controller.get_compute("test1") == compute
    with pytest.raises(aiohttp.web.HTTPNotFound):
        assert controller.get_compute("dsdssd")


def test_has_compute(controller, async_run):
    compute = async_run(controller.add_compute(compute_id="test1", connect=False))

    assert controller.has_compute("test1")
    assert not controller.has_compute("test2")


def test_add_project(controller, async_run):
    uuid1 = str(uuid.uuid4())
    uuid2 = str(uuid.uuid4())

    async_run(controller.add_project(project_id=uuid1, name="Test"))
    assert len(controller.projects) == 1
    async_run(controller.add_project(project_id=uuid1, name="Test"))
    assert len(controller.projects) == 1
    async_run(controller.add_project(project_id=uuid2, name="Test 2"))
    assert len(controller.projects) == 2


def test_addDuplicateProject(controller, async_run):
    uuid1 = str(uuid.uuid4())
    uuid2 = str(uuid.uuid4())

    async_run(controller.add_project(project_id=uuid1, name="Test"))
    assert len(controller.projects) == 1
    with pytest.raises(aiohttp.web.HTTPConflict):
        async_run(controller.add_project(project_id=uuid2, name="Test"))


def test_remove_project(controller, async_run):
    uuid1 = str(uuid.uuid4())

    project1 = async_run(controller.add_project(project_id=uuid1, name="Test"))
    assert len(controller.projects) == 1

    controller.remove_project(project1)
    assert len(controller.projects) == 0


def test_addProject_with_compute(controller, async_run):
    uuid1 = str(uuid.uuid4())

    compute = Compute("test1", controller=MagicMock())
    compute.post = MagicMock()
    controller._computes = {"test1": compute}

    project1 = async_run(controller.add_project(project_id=uuid1, name="Test"))


def test_getProject(controller, async_run):
    uuid1 = str(uuid.uuid4())

    project = async_run(controller.add_project(project_id=uuid1, name="Test"))
    assert controller.get_project(uuid1) == project
    with pytest.raises(aiohttp.web.HTTPNotFound):
        assert controller.get_project("dsdssd")


def test_start(controller, async_run):
    controller.gns3vm.settings = {
        "enable": False,
        "engine": "vmware",
        "vmname": "GNS3 VM"
    }
    with asyncio_patch("gns3server.controller.compute.Compute.connect") as mock:
        async_run(controller.start())
    assert len(controller.computes) == 1  # Local compute is created
    assert controller.computes["local"].name == socket.gethostname()


def test_start_vm(controller, async_run):
    """
    Start the controller with a GNS3 VM
    """
    controller.gns3vm.settings = {
        "enable": True,
        "engine": "vmware",
        "vmname": "GNS3 VM"
    }
    with asyncio_patch("gns3server.controller.gns3vm.vmware_gns3_vm.VMwareGNS3VM.start") as mock:
        with asyncio_patch("gns3server.controller.gns3vm.GNS3VM._check_network") as mock_check_network:
            with asyncio_patch("gns3server.controller.compute.Compute.connect") as mock_connect:
                async_run(controller.start())
                assert mock.called
    assert "local" in controller.computes
    assert "vm" in controller.computes
    assert len(controller.computes) == 2  # Local compute and vm are created


def test_stop(controller, async_run):
    c = async_run(controller.add_compute(compute_id="test1", connect=False))
    c._connected = True
    async_run(controller.stop())
    assert c.connected is False


def test_stop_vm(controller, async_run):
    """
    Stop GNS3 VM if configured
    """
    controller.gns3vm.settings = {
        "enable": True,
        "engine": "vmware",
        "when_exit": "stop",
        "vmname": "GNS3 VM"
    }
    controller.gns3vm.current_engine().running = True
    with asyncio_patch("gns3server.controller.gns3vm.vmware_gns3_vm.VMwareGNS3VM.stop") as mock:
        async_run(controller.stop())
        assert mock.called


def test_suspend_vm(controller, async_run):
    """
    Suspend GNS3 VM if configured
    """
    controller.gns3vm.settings = {
        "enable": True,
        "engine": "vmware",
        "when_exit": "suspend",
        "vmname": "GNS3 VM"
    }
    controller.gns3vm.current_engine().running = True
    with asyncio_patch("gns3server.controller.gns3vm.vmware_gns3_vm.VMwareGNS3VM.suspend") as mock:
        async_run(controller.stop())
        assert mock.called


def test_keep_vm(controller, async_run):
    """
    Keep GNS3 VM if configured
    """
    controller.gns3vm.settings = {
        "enable": True,
        "engine": "vmware",
        "when_exit": "keep",
        "vmname": "GNS3 VM"
    }
    controller.gns3vm.current_engine().running = True
    with asyncio_patch("gns3server.controller.gns3vm.vmware_gns3_vm.VMwareGNS3VM.suspend") as mock:
        async_run(controller.stop())
        assert not mock.called


def test_get_free_project_name(controller, async_run):

    async_run(controller.add_project(project_id=str(uuid.uuid4()), name="Test"))
    assert controller.get_free_project_name("Test") == "Test-1"
    async_run(controller.add_project(project_id=str(uuid.uuid4()), name="Test-1"))
    assert controller.get_free_project_name("Test") == "Test-2"
    assert controller.get_free_project_name("Hello") == "Hello"


def test_load_base_files(controller, config, tmpdir):
    config.set_section_config("Server", {"configs_path": str(tmpdir)})

    with open(str(tmpdir / 'iou_l2_base_startup-config.txt'), 'w+') as f:
        f.write('test')

    controller.load_base_files()
    assert os.path.exists(str(tmpdir / 'iou_l3_base_startup-config.txt'))

    # Check is the file has not been overwrite
    with open(str(tmpdir / 'iou_l2_base_startup-config.txt')) as f:
        assert f.read() == 'test'


def test_appliance_templates(controller, async_run, tmpdir):
    my_appliance = {
        "name": "My Appliance",
        "status": "stable"
    }
    with open(str(tmpdir / "my_appliance.gns3a"), 'w+') as f:
        json.dump(my_appliance, f)
    # A broken appliance
    my_appliance = {
        "name": "Broken"
    }
    with open(str(tmpdir / "my_appliance2.gns3a"), 'w+') as f:
        json.dump(my_appliance, f)

    with patch("gns3server.config.Config.get_section_config", return_value={"appliances_path": str(tmpdir)}):
        controller.load_appliances()
    assert len(controller.appliance_templates) > 0
    for appliance in controller.appliance_templates.values():
        assert appliance.__json__()["status"] != "broken"
    assert "Alpine Linux" in [c.__json__()["name"] for c in controller.appliance_templates.values()]
    assert "My Appliance" in [c.__json__()["name"] for c in controller.appliance_templates.values()]

    for c in controller.appliance_templates.values():
        j = c.__json__()
        if j["name"] == "Alpine Linux":
            assert j["builtin"]
        elif j["name"] == "My Appliance":
            assert not j["builtin"]


def test_load_appliances(controller):
    controller._settings = {
        "Qemu": {
            "vms": [
                {
                    "name": "Test",
                    "node_type": "qemu",
                    "category": "router"
                }
            ]
        }
    }
    controller.load_appliances()
    assert "Test" in [appliance.name for appliance in controller.appliances.values()]
    assert "Cloud" in [appliance.name for appliance in controller.appliances.values()]
    assert "VPCS" in [appliance.name for appliance in controller.appliances.values()]

    for appliance in controller.appliances.values():
        if appliance.name == "VPCS":
            assert appliance._data["properties"] == {"base_script_file": "vpcs_base_config.txt"}

    # UUID should not change when you run again the function
    for appliance in controller.appliances.values():
        if appliance.name == "Test":
            qemu_uuid = appliance.id
        elif appliance.name == "Cloud":
            cloud_uuid = appliance.id
    controller.load_appliances()
    for appliance in controller.appliances.values():
        if appliance.name == "Test":
            assert qemu_uuid == appliance.id
        elif appliance.name == "Cloud":
            assert cloud_uuid == appliance.id


def test_load_appliances_deprecated_features_default_symbol(controller):
    controller._settings = {
        "Qemu": {
            "vms": [
                {
                    "name": "Test",
                    "node_type": "qemu",
                    "category": "router",
                    "default_symbol": ":/symbols/iosv_virl.normal.svg",
                    "hover_symbol": ":/symbols/iosv_virl.selected.svg",
                }
            ]
        }
    }
    controller.load_appliances()
    appliances = dict([(a.name, a) for a in controller.appliances.values()])

    assert appliances["Test"].__json__()["symbol"] == ":/symbols/computer.svg"
    assert "default_symbol" not in appliances["Test"].data.keys()
    assert "hover_symbol" not in appliances["Test"].data.keys()


def test_load_appliances_deprecated_features_default_symbol_with_symbol(controller):
    controller._settings = {
        "Qemu": {
            "vms": [
                {
                    "name": "Test",
                    "node_type": "qemu",
                    "category": "router",
                    "default_symbol": ":/symbols/iosv_virl.normal.svg",
                    "hover_symbol": ":/symbols/iosv_virl.selected.svg",
                    "symbol": ":/symbols/my-symbol.svg"

                }
            ]
        }
    }
    controller.load_appliances()
    appliances = dict([(a.name, a) for a in controller.appliances.values()])

    assert appliances["Test"].__json__()["symbol"] == ":/symbols/my-symbol.svg"
    assert "default_symbol" not in appliances["Test"].data.keys()
    assert "hover_symbol" not in appliances["Test"].data.keys()


def test_autoidlepc(controller, async_run):
    controller._computes["local"] = AsyncioMagicMock()
    node_mock = AsyncioMagicMock()
    with asyncio_patch("gns3server.controller.Project.add_node", return_value=node_mock):
        async_run(controller.autoidlepc("local", "c7200", "test.bin", 512))
    assert node_mock.dynamips_auto_idlepc.called
    assert len(controller.projects) == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import argparse
import shutil
import ipaddress

if sys.platform.startswith("win"):
    import wmi
else:
    raise SystemExit("This script must run on Windows!")


def parse_add_loopback():
    """
    Validate params when adding a loopback adapter
    """

    class Add(argparse.Action):

        def __call__(self, parser, args, values, option_string=None):
            try:
                ipaddress.IPv4Interface("{}/{}".format(values[1], values[2]))
            except ipaddress.AddressValueError as e:
                raise argparse.ArgumentTypeError("Invalid IP address: {}".format(e))
            except ipaddress.NetmaskValueError as e:
                raise argparse.ArgumentTypeError("Invalid subnet mask: {}".format(e))
            setattr(args, self.dest, values)
    return Add


def add_loopback(devcon_path, name, ip_address, netmask):

    # save the list of network adapter in order to find the one we are about to add
    previous_adapters = wmi.WMI().Win32_NetworkAdapter()
    for adapter in previous_adapters:
        if "Loopback" in adapter.Description and adapter.NetConnectionID == name:
            raise SystemExit('Windows loopback adapter named "{}" already exists'.format(name))

    # install a new Windows loopback adapter
    os.system('"{}" install {}\\inf\\netloop.inf *MSLOOP'.format(devcon_path, os.path.expandvars("%WINDIR%")))

    # configure the new Windows loopback adapter
    for adapter in wmi.WMI().Win32_NetworkAdapter():
        if "Loopback" in adapter.Description and adapter not in previous_adapters:
            print('Renaming loopback adapter "{}" to "{}"'.format(adapter.NetConnectionID, name))
            adapter.NetConnectionID = name
            for network_config in wmi.WMI().Win32_NetworkAdapterConfiguration(IPEnabled=True):
                if network_config.InterfaceIndex == adapter.InterfaceIndex:
                    print('Configuring loopback adapter "{}" with {} {}'.format(name, ip_address, netmask))
                    retcode = network_config.EnableStatic(IPAddress=[ip_address], SubnetMask=[netmask])[0]
                    if retcode == 1:
                        print("A reboot is required")
                    elif retcode != 0:
                        print('Error while configuring IP/Subnet mask on "{}"')

                    #FIXME: support gateway?
                    #network_config.SetGateways(DefaultIPGateway=[""])
            break

    # restart winpcap/npcap services to take the new adapter into account
    os.system("net stop npf")
    os.system("net start npf")
    os.system("net stop npcap")
    os.system("net start npcap")


def remove_loopback(devcon_path, name):

    deleted = False
    for adapter in wmi.WMI().Win32_NetworkAdapter():
        if "Loopback" in adapter.Description and adapter.NetConnectionID == name:
            # remove a Windows loopback adapter
            print('Removing loopback adapter "{}"'.format(name))
            os.system('"{}" remove @{}'.format(devcon_path, adapter.PNPDeviceID))
            deleted = True

    if not deleted:
        raise SystemExit('Could not find adapter "{}"'.format(name))

    # update winpcap/npcap services
    os.system("net stop npf")
    os.system("net start npf")
    os.system("net stop npcap")
    os.system("net start npcap")


def main():
    """
    Entry point for the Windows loopback tool.
    """

    parser = argparse.ArgumentParser(description='%(prog)s add/remove Windows loopback adapters')
    parser.add_argument('-a', "--add", nargs=3, action=parse_add_loopback(), help="add a Windows loopback adapter")
    parser.add_argument("-r", "--remove", action="store", help="remove a Windows loopback adapter")
    try:
        args = parser.parse_args()
    except argparse.ArgumentTypeError as e:
        raise SystemExit(e)

    # devcon is required to install/remove Windows loopback adapters
    devcon_path = shutil.which("devcon")
    if not devcon_path:
        raise SystemExit("Could not find devcon.exe")

    from win32com.shell import shell
    if not shell.IsUserAnAdmin():
        raise SystemExit("You must run this script as an administrator")

    try:
        if args.add:
            add_loopback(devcon_path, args.add[0], args.add[1], args.add[2])
        if args.remove:
            remove_loopback(devcon_path, args.remove)
    except SystemExit as e:
        print(e)
        os.system("pause")

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Copyright (C) 2017 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio

from gns3server.utils.asyncio.embed_shell import EmbedShell


def test_embed_shell_help(async_run):
    class Application(EmbedShell):

        @asyncio.coroutine
        def hello(self):
            """
            The hello world function

            The hello usage
            """
            pass
    reader = asyncio.StreamReader()
    writer = asyncio.StreamReader()
    app = Application(reader, writer)
    assert async_run(app._parse_command('help')) == 'Help:\nhello: The hello world function\nhelp: Show help\n\nhelp command for details about a command\n'
    assert async_run(app._parse_command('?')) == 'Help:\nhello: The hello world function\nhelp: Show help\n\nhelp command for details about a command\n'
    assert async_run(app._parse_command('? hello')) == 'hello: The hello world function\n\nThe hello usage\n'


def test_embed_shell_execute(async_run):
    class Application(EmbedShell):

        @asyncio.coroutine
        def hello(self):
            """
            The hello world function

            The hello usage
            """
            return 'world'
    reader = asyncio.StreamReader()
    writer = asyncio.StreamReader()
    app = Application(reader, writer)
    assert async_run(app._parse_command('hello')) == 'world'


def test_embed_shell_welcome(async_run, loop):
    reader = asyncio.StreamReader()
    writer = asyncio.StreamReader()
    app = EmbedShell(reader, writer, welcome_message="Hello")
    t = loop.create_task(app.run())
    assert async_run(writer.read(5)) == b"Hello"
    t.cancel()


def test_embed_shell_prompt(async_run, loop):
    reader = asyncio.StreamReader()
    writer = asyncio.StreamReader()
    app = EmbedShell(reader, writer)
    app.prompt = "gbash# "
    t = loop.create_task(app.run())
    assert async_run(writer.read(7)) == b"gbash# "
    t.cancel()



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import uuid
import html
import asyncio
import aiohttp

from gns3server.utils.asyncio import asyncio_ensure_future

import logging
log = logging.getLogger(__name__)


FILTERS = [
    {
        "type": "frequency_drop",
        "name": "Frequency drop",
        "description": "It will drop everything with a -1 frequency, drop every Nth packet with a positive frequency, or drop nothing",
        "parameters": [
            {
                "name": "Frequency",
                "minimum": -1,
                "maximum": 32767,
                "type": "int",
                "unit": "th packet"
            }
        ]
    },
    {
        "type": "packet_loss",
        "name": "Packet loss",
        "description": "The percentage represents the chance for a packet to be lost",
        "parameters": [
            {
                "name": "Chance",
                "minimum": 0,
                "maximum": 100,
                "type": "int",
                "unit": "%"
            }
        ]
    },
    {
        "type": "delay",
        "name": "Delay",
        "description": "Delay packets in milliseconds. You can add jitter in milliseconds (+/-) of the delay",
        "parameters": [
            {
                "name": "Latency",
                "minimum": 0,
                "maximum": 32767,
                "unit": "ms",
                "type": "int"
            },
            {
                "name": "Jitter (-/+)",
                "minimum": 0,
                "maximum": 32767,
                "unit": "ms",
                "type": "int"
            }
        ]
    },
    {
        "type": "corrupt",
        "name": "Corrupt",
        "description": "The percentage represents the chance for a packet to be corrupted",
        "parameters": [
            {
                "name": "Chance",
                "minimum": 0,
                "maximum": 100,
                "unit": "%",
                "type": "int"
            }
        ]
    },
    {
        "type": "bpf",
        "name": "Berkeley Packet Filter (BPF)",
        "description": "This filter will drop any packet matching a BPF expression. Put one expression per line",
        "parameters": [
            {
                "name": "Filters",
                "type": "text"
            }
        ]
    }
]


class Link:
    """
    Base class for links.
    """

    def __init__(self, project, link_id=None):

        if link_id:
            self._id = link_id
        else:
            self._id = str(uuid.uuid4())
        self._nodes = []
        self._project = project
        self._capturing = False
        self._capture_file_name = None
        self._streaming_pcap = None
        self._created = False
        self._link_type = "ethernet"
        self._suspend = False
        self._filters = {}

    @property
    def filters(self):
        """
        Get an array of filters
        """
        return self._filters

    @property
    def nodes(self):
        """
        Get the current nodes attached to this link
        """
        return self._nodes

    def get_active_filters(self):
        """
        Return the active filters.
        Filters are overridden if the link is suspended.
        """
        if self._suspend:
            return {"frequency_drop": [-1]}
        return self._filters

    @asyncio.coroutine
    def update_filters(self, filters):
        """
        Modify the filters list.

        Filter with value 0 will be dropped because not active
        """
        new_filters = {}
        for (filter, values) in filters.items():
            new_values = []
            for value in values:
                if isinstance(value, str):
                    new_values.append(value.strip("\n "))
                else:
                    new_values.append(int(value))
            values = new_values
            if len(values) != 0 and values[0] != 0 and values[0] != '':
                new_filters[filter] = values

        if new_filters != self.filters:
            self._filters = new_filters
            if self._created:
                yield from self.update()
                self._project.controller.notification.emit("link.updated", self.__json__())
                self._project.dump()

    @asyncio.coroutine
    def update_suspend(self, value):
        if value != self._suspend:
            self._suspend = value
            yield from self.update()
            self._project.controller.notification.emit("link.updated", self.__json__())
            self._project.dump()

    @property
    def created(self):
        """
        :returns: True the link has been created on the computes
        """
        return self._created

    @asyncio.coroutine
    def add_node(self, node, adapter_number, port_number, label=None, dump=True):
        """
        Add a node to the link

        :param dump: Dump project on disk
        """

        port = node.get_port(adapter_number, port_number)
        if port is None:
            raise aiohttp.web.HTTPNotFound(text="Port {}/{} for {} not found".format(adapter_number, port_number, node.name))
        if port.link is not None:
            raise aiohttp.web.HTTPConflict(text="Port is already used")

        self._link_type = port.link_type

        for other_node in self._nodes:
            if other_node["node"] == node:
                raise aiohttp.web.HTTPConflict(text="Cannot connect to itself")

            if node.node_type in ["nat", "cloud"]:
                if other_node["node"].node_type in ["nat", "cloud"]:
                    raise aiohttp.web.HTTPConflict(text="It's not allowed to connect a {} to a {}".format(other_node["node"].node_type, node.node_type))

            # Check if user is not connecting serial => ethernet
            other_port = other_node["node"].get_port(other_node["adapter_number"], other_node["port_number"])
            if other_port is None:
                raise aiohttp.web.HTTPNotFound(text="Port {}/{} for {} not found".format(other_node["adapter_number"], other_node["port_number"], other_node["node"].name))
            if port.link_type != other_port.link_type:
                raise aiohttp.web.HTTPConflict(text="It's not allowed to connect a {} to a {}".format(other_port.link_type, port.link_type))

        if label is None:
            label = {
                "x": -10,
                "y": -10,
                "rotation": 0,
                "text": html.escape("{}/{}".format(adapter_number, port_number)),
                "style": "font-size: 10; font-style: Verdana"
            }

        self._nodes.append({
            "node": node,
            "adapter_number": adapter_number,
            "port_number": port_number,
            "port": port,
            "label": label
        })

        if len(self._nodes) == 2:
            yield from self.create()
            for n in self._nodes:
                n["node"].add_link(self)
                n["port"].link = self
            self._created = True
            self._project.controller.notification.emit("link.created", self.__json__())

        if dump:
            self._project.dump()

    @asyncio.coroutine
    def update_nodes(self, nodes):
        for node_data in nodes:
            node = self._project.get_node(node_data["node_id"])
            for port in self._nodes:
                if port["node"] == node:
                    label = node_data.get("label")
                    if label:
                        port["label"] = label
        self._project.controller.notification.emit("link.updated", self.__json__())
        self._project.dump()

    @asyncio.coroutine
    def create(self):
        """
        Create the link
        """

        raise NotImplementedError

    @asyncio.coroutine
    def update(self):
        """
        Update a link
        """
        raise NotImplementedError

    @asyncio.coroutine
    def delete(self):
        """
        Delete the link
        """
        for n in self._nodes:
            # It could be different of self if we rollback an already existing link
            if n["port"].link == self:
                n["port"].link = None
                n["node"].remove_link(self)

    @asyncio.coroutine
    def start_capture(self, data_link_type="DLT_EN10MB", capture_file_name=None):
        """
        Start capture on the link

        :returns: Capture object
        """

        self._capturing = True
        self._capture_file_name = capture_file_name
        self._streaming_pcap = asyncio_ensure_future(self._start_streaming_pcap())
        self._project.controller.notification.emit("link.updated", self.__json__())

    @asyncio.coroutine
    def _start_streaming_pcap(self):
        """
        Dump a pcap file on disk
        """

        if os.path.exists(self.capture_file_path):
            try:
                os.remove(self.capture_file_path)
            except OSError as e:
                raise aiohttp.web.HTTPConflict(text="Could not delete old capture file '{}': {}".format(self.capture_file_path, e))

        try:
            stream_content = yield from self.read_pcap_from_source()
        except aiohttp.web.HTTPException as e:
            error_msg = "Could not stream PCAP file: error {}: {}".format(e.status, e.text)
            log.error(error_msg)
            self._capturing = False
            self._project.notification.emit("log.error", {"message": error_msg})
            self._project.controller.notification.emit("link.updated", self.__json__())

        with stream_content as stream:
            try:
                with open(self.capture_file_path, "wb") as f:
                    while self._capturing:
                        # We read 1 bytes by 1 otherwise the remaining data is not read if the traffic stops
                        data = yield from stream.read(1)
                        if data:
                            f.write(data)
                            # Flush to disk otherwise the live is not really live
                            f.flush()
                        else:
                            break
            except OSError as e:
                raise aiohttp.web.HTTPConflict(text="Could not write capture file '{}': {}".format(self.capture_file_path, e))

    @asyncio.coroutine
    def stop_capture(self):
        """
        Stop capture on the link
        """

        self._capturing = False
        self._project.controller.notification.emit("link.updated", self.__json__())

    @asyncio.coroutine
    def _read_pcap_from_source(self):
        """
        Return a FileStream of the Pcap from the compute server
        """

        raise NotImplementedError

    @asyncio.coroutine
    def node_updated(self, node):
        """
        Called when a node member of the link is updated
        """
        raise NotImplementedError

    def default_capture_file_name(self):
        """
        :returns: File name for a capture on this link
        """

        capture_file_name = "{}_{}-{}_to_{}_{}-{}".format(self._nodes[0]["node"].name,
                                                          self._nodes[0]["adapter_number"],
                                                          self._nodes[0]["port_number"],
                                                          self._nodes[1]["node"].name,
                                                          self._nodes[1]["adapter_number"],
                                                          self._nodes[1]["port_number"])
        return re.sub("[^0-9A-Za-z_-]", "", capture_file_name) + ".pcap"

    @property
    def id(self):
        return self._id

    @property
    def nodes(self):
        return [node['node'] for node in self._nodes]

    @property
    def capturing(self):
        return self._capturing

    @property
    def capture_file_path(self):
        """
        Get the path of the capture
        """

        if self._capture_file_name:
            return os.path.join(self._project.captures_directory, self._capture_file_name)
        else:
            return None

    def available_filters(self):
        """
        Return the list of filters compatible with this link

        :returns: Array of filters
        """
        filter_node = self._get_filter_node()
        if filter_node:
            return FILTERS
        return []

    def _get_filter_node(self):
        """
        Return the node where the filter will run

        :returns: None if no node support filtering else the node
        """
        for node in self._nodes:
            if node["node"].node_type in ('vpcs',
                                          'traceng',
                                          'vmware',
                                          'dynamips',
                                          'qemu',
                                          'iou',
                                          'cloud',
                                          'nat',
                                          'virtualbox',
                                          'docker'):
                return node["node"]
        return None

    def __eq__(self, other):
        if not isinstance(other, Link):
            return False
        return self.id == other.id

    def __hash__(self):
        return hash(self._id)

    def __json__(self, topology_dump=False):
        """
        :param topology_dump: Filter to keep only properties require for saving on disk
        """
        res = []
        for side in self._nodes:
            res.append({
                "node_id": side["node"].id,
                "adapter_number": side["adapter_number"],
                "port_number": side["port_number"],
                "label": side["label"]
            })
        if topology_dump:
            return {
                "nodes": res,
                "link_id": self._id,
                "filters": self._filters,
                "suspend": self._suspend
            }
        return {
            "nodes": res,
            "link_id": self._id,
            "project_id": self._project.id,
            "capturing": self._capturing,
            "capture_file_name": self._capture_file_name,
            "capture_file_path": self.capture_file_path,
            "link_type": self._link_type,
            "filters": self._filters,
            "suspend": self._suspend
        }



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pytest
from unittest.mock import patch, MagicMock

from gns3server.controller.project import Project
from gns3server.controller.snapshot import Snapshot

from tests.utils import AsyncioMagicMock


@pytest.fixture
def project(controller):
    project = Project(controller=controller, name="Test")
    controller._projects[project.id] = project
    return project


def test_snapshot_name(project):
    """
    Test create a snapshot object with a name
    """
    snapshot = Snapshot(project, name="test1")
    assert snapshot.name == "test1"
    assert snapshot._created_at > 0
    assert snapshot.path.startswith(os.path.join(project.path, "snapshots", "test1_"))
    assert snapshot.path.endswith(".gns3project")

    # Check if UTC conversion doesn't corrupt the path
    snap2 = Snapshot(project, filename=os.path.basename(snapshot.path))
    assert snap2.path == snapshot.path


def test_snapshot_filename(project):
    """
    Test create a snapshot object with a filename
    """
    snapshot = Snapshot(project, filename="test1_260716_100439.gns3project")
    assert snapshot.name == "test1"
    assert snapshot._created_at == 1469527479.0
    assert snapshot.path == os.path.join(project.path, "snapshots", "test1_260716_100439.gns3project")


def test_json(project):
    snapshot = Snapshot(project, filename="test1_260716_100439.gns3project")
    assert snapshot.__json__() == {
        "snapshot_id": snapshot._id,
        "name": "test1",
        "project_id": project.id,
        "created_at": 1469527479
    }


def test_restore(project, controller, async_run):
    compute = AsyncioMagicMock()
    compute.id = "local"
    controller._computes["local"] = compute
    response = AsyncioMagicMock()
    response.json = {"console": 2048}
    compute.post = AsyncioMagicMock(return_value=response)

    async_run(project.add_node(compute, "test1", None, node_type="vpcs", properties={"startup_config": "test.cfg"}))

    snapshot = async_run(project.snapshot(name="test"))

    # We add a node after the snapshots
    async_run(project.add_node(compute, "test2", None, node_type="vpcs", properties={"startup_config": "test.cfg"}))

    # project-files should be reset when reimporting
    test_file = os.path.join(project.path, "project-files", "test.txt")
    os.makedirs(os.path.join(project.path, "project-files"))
    open(test_file, "a+").close()

    assert os.path.exists(test_file)
    assert len(project.nodes) == 2

    controller._notification = MagicMock()
    with patch("gns3server.config.Config.get_section_config", return_value={"local": True}):
        async_run(snapshot.restore())

    assert "snapshot.restored" in [c[0][0] for c in controller.notification.emit.call_args_list]
    # project.closed notification should not be send when restoring snapshots
    assert "project.closed" not in [c[0][0] for c in controller.notification.emit.call_args_list]

    project = controller.get_project(project.id)
    assert not os.path.exists(test_file)
    assert len(project.nodes) == 1



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import os
import stat
import sys
import uuid
import aiohttp

from tests.utils import asyncio_patch
from unittest.mock import patch, MagicMock, PropertyMock

pytestmark = pytest.mark.skipif(sys.platform.startswith("win"), reason="Not supported on Windows")


@pytest.fixture
def fake_iou_bin(images_dir):
    """Create a fake IOU image on disk"""

    path = os.path.join(images_dir, "IOU", "iou.bin")
    with open(path, "w+") as f:
        f.write('\x7fELF\x01\x01\x01')
    os.chmod(path, stat.S_IREAD | stat.S_IEXEC)
    return path


@pytest.fixture
def base_params(tmpdir, fake_iou_bin):
    """Return standard parameters"""
    return {"name": "PC TEST 1", "path": "iou.bin"}


@pytest.fixture
def vm(http_compute, project, base_params):
    response = http_compute.post("/projects/{project_id}/iou/nodes".format(project_id=project.id), base_params)
    assert response.status == 201
    return response.json


def startup_config_file(project, vm):
    directory = os.path.join(project.path, "project-files", "iou", vm["node_id"])
    os.makedirs(directory, exist_ok=True)
    return os.path.join(directory, "startup-config.cfg")


def test_iou_create(http_compute, project, base_params):
    response = http_compute.post("/projects/{project_id}/iou/nodes".format(project_id=project.id), base_params)
    assert response.status == 201
    assert response.route == "/projects/{project_id}/iou/nodes"
    assert response.json["name"] == "PC TEST 1"
    assert response.json["project_id"] == project.id
    assert response.json["serial_adapters"] == 2
    assert response.json["ethernet_adapters"] == 2
    assert response.json["ram"] == 256
    assert response.json["nvram"] == 128
    assert response.json["l1_keepalives"] is False


def test_iou_create_with_params(http_compute, project, base_params):
    params = base_params
    params["ram"] = 1024
    params["nvram"] = 512
    params["serial_adapters"] = 4
    params["ethernet_adapters"] = 0
    params["l1_keepalives"] = True
    params["startup_config_content"] = "hostname test"
    params["use_default_iou_values"] = False

    response = http_compute.post("/projects/{project_id}/iou/nodes".format(project_id=project.id), params, example=True)
    assert response.status == 201
    assert response.route == "/projects/{project_id}/iou/nodes"
    assert response.json["name"] == "PC TEST 1"
    assert response.json["project_id"] == project.id
    assert response.json["serial_adapters"] == 4
    assert response.json["ethernet_adapters"] == 0
    assert response.json["ram"] == 1024
    assert response.json["nvram"] == 512
    assert response.json["l1_keepalives"] is True
    assert response.json["use_default_iou_values"] is False

    with open(startup_config_file(project, response.json)) as f:
        assert f.read() == "hostname test"


def test_iou_create_startup_config_already_exist(http_compute, project, base_params):
    """We don't erase a startup-config if already exist at project creation"""

    node_id = str(uuid.uuid4())
    startup_config_file_path = startup_config_file(project, {'node_id': node_id})
    with open(startup_config_file_path, 'w+') as f:
        f.write("echo hello")

    params = base_params
    params["node_id"] = node_id
    params["startup_config_content"] = "hostname test"

    response = http_compute.post("/projects/{project_id}/iou/nodes".format(project_id=project.id), params, example=True)
    assert response.status == 201
    assert response.route == "/projects/{project_id}/iou/nodes"

    with open(startup_config_file(project, response.json)) as f:
        assert f.read() == "echo hello"


def test_iou_get(http_compute, project, vm):
    response = http_compute.get("/projects/{project_id}/iou/nodes/{node_id}".format(project_id=vm["project_id"], node_id=vm["node_id"]), example=True)
    assert response.status == 200
    assert response.route == "/projects/{project_id}/iou/nodes/{node_id}"
    assert response.json["name"] == "PC TEST 1"
    assert response.json["project_id"] == project.id
    assert response.json["serial_adapters"] == 2
    assert response.json["ethernet_adapters"] == 2
    assert response.json["ram"] == 256
    assert response.json["nvram"] == 128
    assert response.json["l1_keepalives"] is False


def test_iou_start(http_compute, vm):
    with asyncio_patch("gns3server.compute.iou.iou_vm.IOUVM.start", return_value=True) as mock:
        response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/start".format(project_id=vm["project_id"], node_id=vm["node_id"]))
        assert mock.called
        assert response.status == 200
        assert response.json["name"] == "PC TEST 1"


def test_iou_start_with_iourc(http_compute, vm, tmpdir):
    body = {"iourc_content": "test"}

    with asyncio_patch("gns3server.compute.iou.iou_vm.IOUVM.start", return_value=True) as mock:
        response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/start".format(project_id=vm["project_id"], node_id=vm["node_id"]), body=body, example=True)
        assert mock.called
        assert response.status == 200

    response = http_compute.get("/projects/{project_id}/iou/nodes/{node_id}".format(project_id=vm["project_id"], node_id=vm["node_id"]))
    assert response.status == 200


def test_iou_stop(http_compute, vm):
    with asyncio_patch("gns3server.compute.iou.iou_vm.IOUVM.stop", return_value=True) as mock:
        response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/stop".format(project_id=vm["project_id"], node_id=vm["node_id"]), example=True)
        assert mock.called
        assert response.status == 204


def test_iou_reload(http_compute, vm):
    with asyncio_patch("gns3server.compute.iou.iou_vm.IOUVM.reload", return_value=True) as mock:
        response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/reload".format(project_id=vm["project_id"], node_id=vm["node_id"]), example=True)
        assert mock.called
        assert response.status == 204


def test_iou_delete(http_compute, vm):
    with asyncio_patch("gns3server.compute.iou.IOU.delete_node", return_value=True) as mock:
        response = http_compute.delete("/projects/{project_id}/iou/nodes/{node_id}".format(project_id=vm["project_id"], node_id=vm["node_id"]), example=True)
        assert mock.called
        assert response.status == 204


def test_iou_update(http_compute, vm, tmpdir, free_console_port, project):
    params = {
        "name": "test",
        "console": free_console_port,
        "ram": 512,
        "nvram": 2048,
        "ethernet_adapters": 4,
        "serial_adapters": 0,
        "l1_keepalives": True,
        "use_default_iou_values": True,
    }
    response = http_compute.put("/projects/{project_id}/iou/nodes/{node_id}".format(project_id=vm["project_id"], node_id=vm["node_id"]), params, example=True)
    assert response.status == 200
    assert response.json["name"] == "test"
    assert response.json["console"] == free_console_port
    assert response.json["ethernet_adapters"] == 4
    assert response.json["serial_adapters"] == 0
    assert response.json["ram"] == 512
    assert response.json["nvram"] == 2048
    assert response.json["l1_keepalives"] is True
    assert response.json["use_default_iou_values"] is True


def test_iou_nio_create_udp(http_compute, vm):
    response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), {"type": "nio_udp",
                                                                                                                                                                  "lport": 4242,
                                                                                                                                                                  "rport": 4343,
                                                                                                                                                                  "rhost": "127.0.0.1"},
                                 example=True)
    assert response.status == 201
    assert response.route == "/projects/{project_id}/iou/nodes/{node_id}/adapters/{adapter_number:\d+}/ports/{port_number:\d+}/nio"
    assert response.json["type"] == "nio_udp"


def test_iou_nio_update_udp(http_compute, vm):
    response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), {"type": "nio_udp",
                                                                                                                                                                  "lport": 4242,
                                                                                                                                                                  "rport": 4343,
                                                                                                                                                                  "rhost": "127.0.0.1"})
    response = http_compute.put("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]),
                                {
                                    "type": "nio_udp",
                                    "lport": 4242,
                                    "rport": 4343,
                                    "rhost": "127.0.0.1",
                                    "filters": {}},
                                example=True)
    assert response.status == 201, response.body.decode()
    assert response.route == "/projects/{project_id}/iou/nodes/{node_id}/adapters/{adapter_number:\d+}/ports/{port_number:\d+}/nio"
    assert response.json["type"] == "nio_udp"


def test_iou_nio_create_ethernet(http_compute, vm, ethernet_device):
    response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), {"type": "nio_ethernet",
                                                                                                                                                                  "ethernet_device": ethernet_device,
                                                                                                                                                                  },
                                 example=True)
    assert response.status == 201
    assert response.route == "/projects/{project_id}/iou/nodes/{node_id}/adapters/{adapter_number:\d+}/ports/{port_number:\d+}/nio"
    assert response.json["type"] == "nio_ethernet"
    assert response.json["ethernet_device"] == ethernet_device


def test_iou_nio_create_ethernet_different_port(http_compute, vm, ethernet_device):
    response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/0/ports/3/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), {"type": "nio_ethernet",
                                                                                                                                                                  "ethernet_device": ethernet_device,
                                                                                                                                                                  },
                                 example=False)
    assert response.status == 201
    assert response.route == "/projects/{project_id}/iou/nodes/{node_id}/adapters/{adapter_number:\d+}/ports/{port_number:\d+}/nio"
    assert response.json["type"] == "nio_ethernet"
    assert response.json["ethernet_device"] == ethernet_device


def test_iou_nio_create_tap(http_compute, vm, ethernet_device):
    with patch("gns3server.compute.base_manager.BaseManager.has_privileged_access", return_value=True):
        response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), {"type": "nio_tap",
                                                                                                                                                                      "tap_device": ethernet_device})
        assert response.status == 201
        assert response.route == "/projects/{project_id}/iou/nodes/{node_id}/adapters/{adapter_number:\d+}/ports/{port_number:\d+}/nio"
        assert response.json["type"] == "nio_tap"


def test_iou_delete_nio(http_compute, vm):
    http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), {"type": "nio_udp",
                                                                                                                                                       "lport": 4242,
                                                                                                                                                       "rport": 4343,
                                                                                                                                                       "rhost": "127.0.0.1"})
    response = http_compute.delete("/projects/{project_id}/iou/nodes/{node_id}/adapters/1/ports/0/nio".format(project_id=vm["project_id"], node_id=vm["node_id"]), example=True)
    assert response.status == 204
    assert response.route == "/projects/{project_id}/iou/nodes/{node_id}/adapters/{adapter_number:\d+}/ports/{port_number:\d+}/nio"


def test_iou_start_capture(http_compute, vm, tmpdir, project):

    with patch("gns3server.compute.iou.iou_vm.IOUVM.is_running", return_value=True) as mock:
        with asyncio_patch("gns3server.compute.iou.iou_vm.IOUVM.start_capture") as start_capture:

            params = {"capture_file_name": "test.pcap", "data_link_type": "DLT_EN10MB"}
            response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/0/ports/0/start_capture".format(project_id=vm["project_id"], node_id=vm["node_id"]), body=params, example=True)

            assert response.status == 200

            assert start_capture.called
            assert "test.pcap" in response.json["pcap_file_path"]


def test_iou_stop_capture(http_compute, vm, tmpdir, project):

    with patch("gns3server.compute.iou.iou_vm.IOUVM.is_running", return_value=True) as mock:
        with asyncio_patch("gns3server.compute.iou.iou_vm.IOUVM.stop_capture") as stop_capture:

            response = http_compute.post("/projects/{project_id}/iou/nodes/{node_id}/adapters/0/ports/0/stop_capture".format(project_id=vm["project_id"], node_id=vm["node_id"]), example=True)

            assert response.status == 204

            assert stop_capture.called


def test_images(http_compute, fake_iou_bin):

    response = http_compute.get("/iou/images", example=True)
    assert response.status == 200
    assert response.json == [{"filename": "iou.bin", "path": "iou.bin", "filesize": 7, "md5sum": "e573e8f5c93c6c00783f20c7a170aa6c"}]


def test_image_vm(http_compute, tmpdir):
    with patch("gns3server.compute.IOU.get_images_directory", return_value=str(tmpdir),):
        response = http_compute.post("/iou/images/test2", body="TEST", raw=True)
        assert response.status == 204

    with open(str(tmpdir / "test2")) as f:
        assert f.read() == "TEST"

    with open(str(tmpdir / "test2.md5sum")) as f:
        checksum = f.read()
        assert checksum == "033bd94b1168d7e4f0d644c3c95e35bf"


def test_iou_duplicate(http_compute, vm):
    with asyncio_patch("gns3server.compute.iou.IOU.duplicate_node", return_value=True) as mock:
        response = http_compute.post(
            "/projects/{project_id}/iou/nodes/{node_id}/duplicate".format(
                project_id=vm["project_id"],
                node_id=vm["node_id"]),
            body={
                "destination_node_id": str(uuid.uuid4())
            },
            example=True)
        assert mock.called
        assert response.status == 201



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from gns3server.utils.picture import get_size


def test_get_size():
    with open("tests/resources/nvram_iou", "rb") as f:
        res = get_size(f.read(), default_width=100, default_height=50)
        assert res == (100, 50, None)
    with open("tests/resources/gns3_icon_128x64.gif", "rb") as f:
        res = get_size(f.read())
        assert res == (128, 64, "gif")
    with open("tests/resources/gns3_icon_128x64.jpg", "rb") as f:
        res = get_size(f.read())
        assert res == (128, 64, "jpg")
    with open("tests/resources/gns3_icon_128x64.png", "rb") as f:
        res = get_size(f.read())
        assert res == (128, 64, "png")
    with open("gns3server/symbols/dslam.svg", "rb") as f:
        res = get_size(f.read())
        assert res == (50, 53, "svg")
    # Symbol using size with cm
    with open("gns3server/symbols/cloud.svg", "rb") as f:
        res = get_size(f.read())
        assert res == (159, 71, "svg")
    # Size with px
    with open("tests/resources/firefox.svg", "rb") as f:
        res = get_size(f.read())
        assert res == (66, 70, "svg")
